@@ -17,13 +17,12 @@
 #include <falcon/syntree.h>
 #include <falcon/src_lexer.h>
 #include <falcon/error.h>
-#include <falcon/deferrorhandler.h>
 #include <falcon/stdstreams.h>
 #include <falcon/itemid.h>
 #include <falcon/fassert.h>
 #include <falcon/path.h>
 #include <falcon/intcomp.h>
-#include <falcon/flcloader.h>
+#include <falcon/modloader.h>
 #include <falcon/vm.h>
 #include <falcon/stringstream.h>
 #include "core_module/core_module.h"
@@ -52,7 +51,6 @@ Compiler::Compiler( Module *mod, Stream* in ):
    m_lexer(0),
    m_stream( in ),
 
-   m_errhand(0),
    m_module(0),
    m_enumId(0),
 
@@ -66,14 +64,13 @@ Compiler::Compiler( Module *mod, Stream* in ):
    m_modVersion( 0 ),
    m_defContext( false ),
    m_bParsingFtd(false),
+   m_bInteractive( false ),
 
-   m_delayRaise( false ),
    m_rootError( 0 ),
    m_metacomp( 0 ),
    m_serviceVM( 0 ),
    m_serviceLoader( 0 )
 {
-   // Initializing now prevents adding predefined constants to the module.
    init();
 
    m_module = mod;
@@ -93,7 +90,6 @@ Compiler::Compiler():
    m_optLevel(0),
    m_lexer(0),
    m_stream( 0 ),
-   m_errhand(0),
    m_module( 0 ),
    m_enumId( 0 ),
    m_staticPrefix(0),
@@ -105,13 +101,12 @@ Compiler::Compiler():
    m_modVersion( 0 ),
    m_defContext( false ),
    m_bParsingFtd(false),
-   m_delayRaise( false ),
+   m_bInteractive( false ),
    m_rootError( 0 ),
    m_metacomp(0),
    m_serviceVM( 0 ),
    m_serviceLoader( 0 )
 {
-   init();
    addPredefs();
 }
 
@@ -162,13 +157,6 @@ void Compiler::reset()
       clear();
    }
 
-   // reset contants
-   /*MapIterator iter = m_constants.begin();
-   while( iter.hasCurrent() ) {
-      delete *(Value **) iter.currentValue();
-      iter.next();
-   }*/
-   //m_constants.clear();
    m_namespaces.clear();
 
    m_alias.clear();
@@ -176,9 +164,6 @@ void Compiler::reset()
    m_func_ctx.clear();
    m_contextSet.clear();
    m_loops.clear();
-   m_statementVals.clear();
-
-   //addPredefs();
 
    m_errors = 0;
    m_enumId = 0;
@@ -195,7 +180,9 @@ void Compiler::reset()
 void Compiler::clear()
 {
 
-   delete m_serviceVM;
+   if( m_serviceVM != 0 )
+      m_serviceVM->finalize();
+
    delete m_serviceLoader;
    delete m_metacomp;
    m_serviceVM = 0;
@@ -263,6 +250,12 @@ bool Compiler::compile()
    // parse
    m_lexer->input( m_stream );
    flc_src_parse( this );
+   if ( ! m_stream->good() )
+   {
+      raiseError( new IoError( ErrorParam( e_io_error, __LINE__ )
+         .origin( e_orig_compiler )
+         .sysError( (uint32) m_stream->lastError() ) ) );
+   }
    m_module->language( m_language );
    m_module->version( (uint32) m_modVersion );
 
@@ -290,18 +283,6 @@ bool Compiler::compile()
       return true;
    }
 
-   if ( m_delayRaise && m_rootError != 0 && m_errhand != 0 )
-   {
-      m_errhand->handleError( m_rootError );
-   }
-
-   // eventually prepare for next compilation
-   if ( m_rootError != 0 )
-   {
-      m_rootError->decref();
-      m_rootError = 0;
-   }
-
    // we had errors.
    return false;
 }
@@ -333,25 +314,24 @@ void Compiler::raiseError( int code, const String &errorp, int line )
    if ( line == 0 )
       line = lexer()->line()-1;
 
-   if ( m_errhand != 0 )
+   SyntaxError *error = new SyntaxError( ErrorParam(code, line)
+         .origin( e_orig_compiler ) );
+   error->extraDescription( errorp );
+   error->module( String(m_module->path()).bufferize() );
+
+   raiseError( error );
+}
+
+void Compiler::raiseError( Error *error )
+{
+   if ( m_rootError == 0 )
+   {
+      m_rootError = error;
+   }
+   else
    {
-      SyntaxError *error = new SyntaxError( ErrorParam(code, line).origin( e_orig_compiler ));
-      error->extraDescription( errorp );
       error->module( m_module->path() );
-
-      if ( m_delayRaise )
-      {
-         if ( m_rootError == 0 )
-         {
-            m_rootError = new SyntaxError( ErrorParam( e_syntax ).origin( e_orig_compiler ) );
-         }
-         m_rootError->appendSubError( error );
-      }
-      else
-      {
-         m_errhand->handleError( error );
-      }
-
+      m_rootError->appendSubError( error );
       error->decref();
    }
 
@@ -447,7 +427,10 @@ void Compiler::defineVal( Value *val )
       if ( staticPrefix() == 0 )
       {
          Symbol *sym;
-         if ( getFunction() != 0 ) {
+         // are we in a function that misses that symbol?
+         if ( getFunction() != 0 )
+         {
+            // we're creating the local symbol
             // addlocal must also define the symbol
             sym = addLocalSymbol( val->asSymdef(), false );
          }
@@ -501,22 +484,35 @@ Symbol *Compiler::addLocalSymbol( const String *symname, bool parameter )
    Symbol *sym = table.findByName( *symname );
    if( sym == 0 )
    {
-      if ( m_strict && ! m_defContext )
-      {
-         raiseError( e_undef_sym, "", lexer()->previousLine() );
-      }
-
       // now we can add the symbol. As we have the string from
       // the module already, we keep it.
       sym = new Symbol( m_module, symname );
       m_module->addSymbol( sym );
       sym->declaredAt( lexer()->previousLine() );
+
+      // this flag marks closure forward definitions
+      bool taken = false;
       if ( parameter ) {
          sym = func->addParameter( sym );
       }
       else
       {
-         sym = func->addLocal( sym );
+         // If we're in a closure, we may wish to add
+         // a local undefined that will be filled at closure ending
+         if ( m_closureContexts && searchLocalSymbol( symname, true ) != 0 )
+         {
+            taken = true;
+            sym = func->addUndefined( sym );
+         }
+         else
+            sym = func->addLocal( sym );
+      }
+
+      // in strict mode raise an error if we're not in def, but not if
+      // we taken the symbol from some parent.
+      if ( !taken && m_strict && !m_defContext )
+      {
+         raiseError( e_undef_sym, "", lexer()->previousLine() );
       }
    }
    return sym;
@@ -532,21 +528,34 @@ bool Compiler::checkLocalUndefined()
       if ( val->isSymdef() )
       {
          Symbol *sym = 0;
+
          if ( m_closureContexts > 0 )
          {
-            fassert( m_functions.end() );
-            if ( m_functions.begin() != m_functions.end() )
-            {
-               const FuncDef *fd_parent = reinterpret_cast<const FuncDef *>
-                  ( m_functions.end()->prev()->data() );
+            // still undefined after some loop?
+            // in case of undef == x[undef.len()] we have double
+            // un-definition of undef on the same line.
+            fassert( m_functions.end() != 0 );
+            ListElement* le = m_functions.end();
+            const FuncDef *fd_current = reinterpret_cast<const FuncDef *>( le->data() );
 
-               if( fd_parent != 0 )
+            // try to find it locally -- may have been defined in a previous loop
+            sym = fd_current->symtab().findByName( *val->asSymdef() );
+            if  ( sym == 0 )
+            {
+               le = le->prev();
+               while( le != 0 )
                {
+                  const FuncDef *fd_parent = reinterpret_cast<const FuncDef *>( le->data() );
+
                   if( fd_parent->symtab().findByName( *val->asSymdef() ) != 0 )
                   {
-                     sym = getFunction()->addUndefined(
-                        m_module->addSymbol(*val->asSymdef()) );
+                     sym = m_module->addSymbol(*val->asSymdef());
+                     sym->declaredAt( lexer()->previousLine() );
+                     getFunction()->addUndefined( sym );
+                     break;
                   }
+
+                  le = le->prev();
                }
             }
          }
@@ -554,17 +563,20 @@ bool Compiler::checkLocalUndefined()
          // still nothing?
          if (sym == 0)
          {
+            // --- import the symbol
             sym = addGlobalSymbol( val->asSymdef() );
          }
+
          val->setSymbol( sym );
       }
+
       l->popFront();
    }
    return true;
 }
 
 
-Symbol *Compiler::searchLocalSymbol( const String *symname )
+Symbol *Compiler::searchLocalSymbol( const String *symname, bool recurse )
 {
    if( m_functions.empty() )
       return searchGlobalSymbol( symname );
@@ -576,8 +588,22 @@ Symbol *Compiler::searchLocalSymbol( const String *symname )
       return *sympp;
 
    // then try in the local symtab or just return 0.
-   FuncDef *fd = (FuncDef *) m_functions.back();
-   return fd->symtab().findByName( *symname );
+   ListElement* lastFunc = m_functions.end();
+   while( lastFunc != 0 )
+   {
+      FuncDef *fd = (FuncDef *)lastFunc->data();
+      Symbol *found;
+      if( (found = fd->symtab().findByName( *symname )) )
+         return found;
+
+      if ( ! recurse )
+         return 0;
+
+      lastFunc = lastFunc->prev();
+   }
+
+   // definitely not found.
+   return 0;
 }
 
 Symbol *Compiler::searchOuterSymbol( const String *symname )
@@ -666,27 +692,6 @@ Symbol *Compiler::addGlobalVar( const String *symname, VarDef *value )
    return sym;
 }
 
-
-Symbol *Compiler::addAttribute( const String *symname )
-{
-   // find the global symbol for this.
-   Symbol *sym = searchGlobalSymbol( symname );
-
-   // Not defined?
-   if( sym == 0 ) {
-      sym = addGlobalSymbol( symname );
-      sym->declaredAt( lexer()->previousLine() );
-   }
-   else {
-      raiseError( e_already_def,  sym->name() );
-   }
-   // but change it in an attribute anyhow
-   sym->setAttribute();
-
-   return sym;
-}
-
-
 Symbol *Compiler::globalize( const String *symname )
 {
    if ( ! isLocalContext() ) {
@@ -772,10 +777,9 @@ void Compiler::addPredefs()
 
    addIntConstant( "NilType", FLC_ITEM_NIL );
    addIntConstant( "BooleanType", FLC_ITEM_BOOL );
-   addIntConstant( "IntegerType", FLC_ITEM_INT );
+   //addIntConstant( "IntegerType", FLC_ITEM_INT );
    addIntConstant( "NumericType", FLC_ITEM_NUM );
    addIntConstant( "RangeType", FLC_ITEM_RANGE );
-   addIntConstant( "AttributeType", FLC_ITEM_ATTRIBUTE );
    addIntConstant( "FunctionType", FLC_ITEM_FUNC );
    addIntConstant( "StringType", FLC_ITEM_STRING );
    addIntConstant( "LBindType", FLC_ITEM_LBIND );
@@ -785,9 +789,22 @@ void Compiler::addPredefs()
    addIntConstant( "ObjectType", FLC_ITEM_OBJECT );
    addIntConstant( "ClassType", FLC_ITEM_CLASS );
    addIntConstant( "MethodType", FLC_ITEM_METHOD );
-   addIntConstant( "TableMethodType", FLC_ITEM_TABMETHOD );
    addIntConstant( "ClassMethodType", FLC_ITEM_CLSMETHOD );
-  
+
+}
+
+
+void Compiler::addAttribute( const String &name, Value *val, uint32 line )
+{
+   String n = name;
+   FuncDef* fd = getFunction();
+   if( fd == 0 )
+   {
+      m_module->addAttribute( name, val->genVarDef() );
+   }
+   else {
+      fd->addAttrib(name, val->genVarDef() );
+   }
 }
 
 void Compiler::addIntConstant( const String &name, int64 value, uint32 line )
@@ -980,7 +997,7 @@ Value *Compiler::closeClosure()
    if( fd->undefined() > 0 )
    {
       // we have to find all the local variables that exist in the upper context and
-      // transform them in parameters.
+      // transform them in the first part of the local array.
       SymbolTable &funcTable = fd->symtab();
       ArrayDecl *closureDecl = new ArrayDecl;
 
@@ -994,45 +1011,66 @@ Value *Compiler::closeClosure()
          Symbol *sym = *(Symbol **) iter.currentValue();
          if ( sym->isLocalUndef() )
          {
-            // ok, this must become a parameter...
-            sym->setParam();
+            // ok, this must become a local symbol...
+            sym->setLocal();
             sym->itemId( moved );
             moved ++;
 
-            Symbol *parentSym;
-            if ( (parentSym = searchLocalSymbol( &sym->name() )) != 0 )
+            // and now let's find the father's having this local undefined.
+            fassert( m_functions.end() != 0 );
+            ListElement* le = m_functions.end();
+            Symbol *parentSym = 0;
+
+            while( le != 0 )
             {
-               //... and the parent symbol must be stored in parametric array...
-               closureDecl->pushBack( new Value( parentSym ) );
+               FuncDef *fd_parent = ( FuncDef *) le->data();
+
+               // when we find it...
+               if( ( parentSym = fd_parent->symtab().findByName( sym->name() )) != 0 )
+               {
+                  // we must now force the declaration of local undefined for this
+                  // symbol up to this closure, so that it sent down the stack on runtime.
+                  le = le->next();
+                  while( le != 0 )
+                  {
+                     fd_parent = (FuncDef *) le->data();
+                     parentSym = fd_parent->addUndefined( m_module->addSymbol( sym->name() ) );
+                     le = le->next();
+                  }
+
+                  //... and the parent symbol must be stored in parametric array...
+                  closureDecl->pushBack( new Value( parentSym ) );
+                  break;
+               }
+
+               le = le->prev();
             }
-            else {
+
+            if ( parentSym == 0 )
+            {
                // closures can't have undefs.
                raiseError( e_undef_sym, "", sym->declaredAt() );
             }
          }
-         else if ( sym->isParam() )
+         else if ( sym->isLocal() )
          {
-            // push forward all parameters
+            // push forward all the locals
             sym->itemId( fd->undefined() + sym->itemId() );
          }
 
          iter.next();
       }
 
-      // no more undefs -- now they are params
-      fd->params( fd->params() + fd->undefined() );
+      // no more undefs -- now they are locals
+      fd->locals( fd->locals() + fd->undefined() );
       fd->undefined( 0 );
 
-      // ... put it in front of our array and return it.
-      closureDecl->pushFront( lambda_call );
-      return new Value( closureDecl );
-   }
-   else
-   {
-      // just create create a lambda call for this function
-      return lambda_call;
+      // ... put as third element of the lambda call.
+      lambda_call->asExpr()->second( new Value( closureDecl ) );
    }
 
+   // just create create a lambda call for this function
+   return lambda_call;
 }
 
 
@@ -1156,9 +1194,20 @@ void Compiler::addNamespace( const String &nspace, const String &alias, bool ful
 
       m_module->addDepend( nselfed, nspace, true, isFilename );
    }
-   // no? -- eventually change to load all.
-   else if ( *res == 0 && full )
-      *res = (void *) 1;
+   // no?
+   else {
+      // -- eventually change to load all.
+      if ( *res == 0 && full )
+         *res = (void *) 1;
+
+      // -- and check if this is a mis-redefinition
+      // raise an error if you try to alias a different module with an alredy existing namespace
+      ModuleDepData* mdPrev = m_module->dependencies().findModule( nselfed );
+      if ( mdPrev != 0 && *mdPrev->moduleName() != nspace )
+      {
+         raiseError( e_ns_clash, nselfed, lexer()->previousLine() );
+      }
+   }
 }
 
 
@@ -1245,23 +1294,30 @@ void Compiler::importSymbols( List *lst, const String *prefix, const String *ali
 
 void Compiler::metaCompile( const String &data, int startline )
 {
+   String ioEnc, srcEnc;
+   Engine::getEncodings( srcEnc, ioEnc );
+
    // evantually turn on the meta-compiler
    if ( m_metacomp == 0 )
    {
       if ( m_serviceVM == 0 )
       {
          m_serviceVM = new VMachine;
+         m_serviceVM->appSearchPath( searchPath() );
          Module*cm = core_module_init();
          m_serviceVM->link( cm );
          cm->decref();
       }
 
       if ( m_serviceLoader == 0 )
-         m_serviceLoader = new FlcLoader( "." );
+      {
+         m_serviceLoader = new ModuleLoader( searchPath() );
+         m_serviceLoader->sourceEncoding( srcEnc );
+      }
 
-      m_serviceVM->stdOut( new StringStream );
       m_metacomp = new InteractiveCompiler( m_serviceLoader, m_serviceVM );
-      m_metacomp->errorHandler( errorHandler() );
+      m_metacomp->searchPath( searchPath() );
+
       // not incremental...
       m_metacomp->lexer()->incremental( false );
       // do not confuse our user...
@@ -1282,26 +1338,31 @@ void Compiler::metaCompile( const String &data, int startline )
 
    }
 
+   StringStream* output = new StringStream;
+   // set the same encoding as the source that we're parsing.
+   m_serviceVM->stdOut( TranscoderFactory( srcEnc, output, true ) );
+
+
    // set current line in lexer of the meta compiler
    m_metacomp->tempLine( startline );
-   InteractiveCompiler::t_ret_type ret = m_metacomp->compileAll( data );
+   try
+   {
+      m_metacomp->compileAll( data );
 
-   if ( ret == InteractiveCompiler::e_error || ret ==  InteractiveCompiler::e_vm_error )
-      m_errors++;
-   else {
-      StringStream *ss = static_cast<StringStream *>(m_metacomp->vm()->stdOut());
       // something has been written
-      if ( ss->length() != 0 )
+      if ( output->length() != 0 )
       {
          // pass it to the lexer
-         ss->seekBegin(0);
+         output->seekBegin(0);
          StringStream *transferred = new StringStream();
-         transferred->transfer( *ss );
+         transferred->transfer( *output );
          m_lexer->appendStream( transferred );
-         // and dispose it
-         m_metacomp->vm()->stdOut( new StringStream );
       }
    }
+   catch( Error *e )
+   {
+      raiseError( e );
+   }
 }
 
 }
