@@ -5,7 +5,7 @@
    Short description
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
-   Begin: $DATE
+   Begin:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
@@ -32,7 +32,8 @@ GenCode::c_jmptag::c_jmptag( Stream *stream, uint32 offset ):
    m_elifQueries( &traits::t_List() ),
    m_tries( 0 ),
    m_offset( offset ),
-   m_bIsForIn( false ),
+   m_bIsForLast( false ),
+   m_ForinLoop(0),
    m_stream( stream )
 {
    m_defs[0] = m_defs[1] = m_defs[2] = m_defs[3] = JMPTAG_UNDEFINED;
@@ -81,7 +82,7 @@ void GenCode::c_jmptag::define( uint32 id )
 
    uint32 current_pos = (uint32) m_stream->tell();
    m_defs[id] = current_pos + m_offset;
-   uint32 value = endianInt32( current_pos + m_offset );
+   uint32 value = current_pos + m_offset;
 
    ListElement *iter = m_queries[id].begin();
    while( iter != 0 )
@@ -115,7 +116,7 @@ void GenCode::c_jmptag::defineElif( uint32 id )
    uint32 current_pos = (uint32) m_stream->tell();
    uint32 cp = current_pos + m_offset;
    m_elifDefs.set( &cp, id );
-   uint32 value = endianInt32( current_pos + m_offset );
+   uint32 value = current_pos + m_offset;
    List *lst = (List *) m_elifQueries.at(id);
    ListElement *iter = lst->begin();
    while( iter != 0 )
@@ -254,7 +255,7 @@ void GenCode::c_varpar::generate( GenCode *owner ) const
 
       case e_parSYM:
       {
-         uint32 out = endianInt32( m_content.sym->itemId() );
+         uint32 out = m_content.sym->itemId();
          owner->m_outTemp->write( &out, sizeof(out) );
       }
       break;
@@ -263,14 +264,14 @@ void GenCode::c_varpar::generate( GenCode *owner ) const
       case e_parNTD32:
       case e_parINT32:
       {
-         uint32 out = endianInt32( m_content.immediate );
+         uint32 out = m_content.immediate;
          owner->m_outTemp->write( &out, sizeof(out) );
       }
       break;
 
       case e_parNTD64:
       {
-         uint64 out = endianInt64( m_content.immediate64 );
+         uint64 out = m_content.immediate64;
          owner->m_outTemp->write( &out, sizeof(out) );
       }
       break;
@@ -291,42 +292,41 @@ void GenCode::gen_operand( const Value *stmt )
       {
          uint32 symid;
          const Symbol *sym = stmt->asSymbol();
-         symid = endianInt32( sym->itemId() );
+         symid = sym->itemId();
          m_outTemp->write( &symid, sizeof( symid ) );
       }
       break;
 
       case Value::t_imm_integer:
       {
-         int64 ival = endianInt64( stmt->asInteger() );
+         int64 ival = stmt->asInteger();
          m_outTemp->write( &ival, sizeof( ival ) );
       }
       break;
 
       case Value::t_imm_num:
       {
-         numeric dval = endianNum( stmt->asNumeric() );
+         numeric dval = stmt->asNumeric();
          m_outTemp->write( &dval, sizeof( dval ) );
       }
       break;
 
       case Value::t_imm_string:
       {
-         uint32 strid = (uint32) endianInt32( stmt->asString()->id() );
+         uint32 strid = (uint32) stmt->asString()->id();
          m_outTemp->write( &strid, sizeof( strid ) );
       }
       break;
 
       case Value::t_lbind:
       {
-         uint32 strid = (uint32) endianInt32( stmt->asLBind()->id() );
+         uint32 strid = (uint32) stmt->asLBind()->id();
          m_outTemp->write( &strid, sizeof( strid ) );
       }
       break;
 
       case Value::t_imm_bool:
       case Value::t_self:
-      case Value::t_sender:
          // do nothing
       break;
 
@@ -352,7 +352,6 @@ byte GenCode::gen_pdef( const c_varpar &elem )
             case Value::t_imm_string: return P_PARAM_STRID;
             case Value::t_imm_num: return P_PARAM_NUM;
             case Value::t_self: return P_PARAM_REGS1;
-            case Value::t_sender: return P_PARAM_REGS2;
             case Value::t_lbind: return P_PARAM_LBIND;
 
             case Value::t_symbol:
@@ -410,7 +409,6 @@ byte GenCode::gen_pdef( const c_varpar &elem )
       case e_parL1: return P_PARAM_REGL1;
       case e_parL2: return P_PARAM_REGL2;
       case e_parS1: return P_PARAM_REGS1;
-      case e_parS2: return P_PARAM_REGS2;
       case e_parLBIND: return P_PARAM_LBIND;
       case e_parTRUE: return P_PARAM_TRUE;
       case e_parFALSE: return P_PARAM_FALSE;
@@ -427,21 +425,21 @@ void GenCode::gen_var( const VarDef &def )
    {
       case VarDef::t_int:
       {
-         int64 ival = endianInt64( def.asInteger() );
+         int64 ival = def.asInteger();
          m_outTemp->write( reinterpret_cast< const char *>( &ival ), sizeof( ival ) );
       }
       break;
 
       case VarDef::t_num:
       {
-         numeric num = endianNum( def.asNumeric() );
+         numeric num = def.asNumeric();
          m_outTemp->write( reinterpret_cast< const char *>( &num ), sizeof( num ) );
       }
       break;
 
       case VarDef::t_string:
       {
-         int32 id = endianInt32( def.asString()->id() );
+         int32 id = def.asString()->id();
          m_outTemp->write( reinterpret_cast< const char *>( &id ), sizeof( id ) );
       }
       break;
@@ -449,7 +447,7 @@ void GenCode::gen_var( const VarDef &def )
       case VarDef::t_symbol:
       {
          const Symbol *sym = def.asSymbol();
-         int32 ival = endianInt32( sym->itemId() );
+         int32 ival = sym->itemId();
          m_outTemp->write( reinterpret_cast< const char *>( &ival ), sizeof( ival ) );
       }
       break;
@@ -566,10 +564,50 @@ void GenCode::gen_statement( const Statement *stmt )
             if( tag.isForIn() )
             {
                const StmtContinue *cont = static_cast<const StmtContinue *>( stmt );
-               gen_pcode( P_TRAN,
-                  c_param_fixed( tag.addQueryBegin() ),
-                  c_param_fixed( tag.addQueryElif( 0, 2 ) ),
-                  c_param_fixed( cont->dropping() ? 1 : 0 ) );
+               // last element has a special continue semantic.
+               if( tag.isForLast() )
+               {
+                  // on dropping, just drop the last and go away
+                  if( cont->dropping() )
+                  {
+                     gen_pcode( P_TRDN,
+                       c_param_fixed( tag.addQueryEnd() ),
+                       c_param_fixed( 0 ),
+                       c_param_fixed( 0 ) );
+                  }
+                  else
+                  {
+                     // exactly as a break
+                     jmppos = tag.addQueryEnd();
+                     gen_pcode( P_JMP, c_param_fixed( jmppos ) );
+                  }
+               }
+               else
+               {
+                  if( cont->dropping() )
+                  {
+                     const StmtForin* fi = tag.ForinLoop();
+                     uint32 nv = fi->dest()->size();
+                     gen_pcode( P_TRDN,
+                          c_param_fixed( tag.addQueryBegin() ),
+                          c_param_fixed( tag.addQueryEnd( 2 ) ),
+                          c_param_fixed( nv ) );
+
+                     ListElement* ite = fi->dest()->begin();
+                     while( ite != 0 )
+                     {
+                        Value* v = (Value*) ite->data();
+                        fassert( v->isSimple() );
+                        gen_pcode( P_NOP, v );
+                        ite = ite->next();
+                     }
+                  }
+                  else {
+                     // a normal continue -> next
+                     jmppos = tag.addQueryNext();
+                     gen_pcode( P_JMP, c_param_fixed( jmppos ) );
+                  }
+               }
             }
             else {
                jmppos = tag.addQueryNext();
@@ -594,28 +632,6 @@ void GenCode::gen_statement( const Statement *stmt )
       }
       break;
 
-      case Statement::t_pass:
-      {
-         const StmtPass *pass = static_cast< const StmtPass *>( stmt );
-         byte mode = pass->saveIn() == 0 ? P_PASS : P_PSIN;
-
-         if ( pass->called()->isSimple() )
-         {
-            gen_pcode( mode, pass->called() );
-         }
-         else {
-            gen_value( pass->called() );
-            gen_pcode( mode, e_parA );
-         }
-
-         if ( pass->saveIn() != 0 )
-         {
-            gen_load_from_A( pass->saveIn() );
-         }
-      }
-      break;
-
-
       case Statement::t_autoexp:
       {
          const StmtExpression *val = static_cast<const StmtExpression *>( stmt );
@@ -670,98 +686,9 @@ void GenCode::gen_statement( const Statement *stmt )
             gen_pcode( P_WRT, val );
             iter = iter->next();
          }
-
       }
       break;
 
-      case Statement::t_give:
-      {
-         const StmtGive *give = static_cast<const StmtGive *>( stmt );
-         const ArrayDecl *give_to = give->objects();
-
-         const ListElement *elem = give_to->begin();
-         while( elem != 0 )
-         {
-            const Value *object = (const Value *) elem->data();
-
-            if ( ! object->isSimple() )
-            {
-               gen_complex_value( object );
-            }
-
-            const ArrayDecl *attribs = give->attributes();
-            ListElement *iter = attribs->begin();
-            bool pushed = false;
-            bool peek = false;
-
-            while( iter != 0 )
-            {
-               const Value *val = (const Value *) iter->data();
-               byte mode;
-               ListElement *iter_next = iter->next();
-
-               if ( val->isExpr() && val->asExpr()->type() == Expression::t_not ) {
-                  val = val->asExpr()->first();
-                  mode = P_GIVN;
-               }
-               else
-                  mode = P_GIVE;
-
-
-               if( val->isSimple() )
-               {
-                  if ( object->isSimple() ) {
-                     gen_pcode( mode, object, val );
-                  }
-                  else {
-                     if ( peek ) {
-                        if ( iter_next != 0 )
-                           gen_pcode( P_PEEK, e_parA );
-                        else {
-                           gen_pcode( P_POP, e_parA );
-                           pushed = false;
-                        }
-                        peek = false;
-                     }
-                     gen_pcode( mode, e_parA, val );
-                  }
-               }
-               else {
-                  if ( object->isSimple() ) {
-                     gen_value( val );
-                     gen_pcode( mode, object, e_parA );
-                  }
-                  else {
-                     if ( ! pushed ) {
-                        gen_pcode( P_PUSH, e_parA );
-                        pushed = true;
-                     }
-
-                     gen_value( val );
-                        if ( iter_next != 0 )
-                           gen_pcode( P_PEEK, e_parB );
-                        else {
-                           gen_pcode( P_POP, e_parB );
-                           pushed = false;
-                        }
-                     peek = true;
-
-                     gen_pcode( mode, e_parB, e_parA );
-                  }
-               }
-
-               iter = iter_next;
-            }
-            if ( pushed ) {
-               gen_pcode( P_IPOP, c_param_fixed( 1 ) );
-            }
-
-            elem = elem->next();
-         }
-      }
-      break;
-
-
       case Statement::t_unref:
       {
          const StmtUnref *ass = static_cast<const StmtUnref *>(stmt);
@@ -897,7 +824,7 @@ void GenCode::gen_statement( const Statement *stmt )
          {
             Value *first = *(Value **) iter.currentKey();
             uint32 second = *(int32 *) iter.currentValue();
-            int64 value = endianInt64( first->asInteger() );
+            int64 value = first->asInteger();
             m_outTemp->write( &value, sizeof(value ) );
             tag.addQuerySwitchBlock( second, 0 );
             m_outTemp->write( &dummy, sizeof( dummy ) );
@@ -910,8 +837,8 @@ void GenCode::gen_statement( const Statement *stmt )
          {
             Value *first = *(Value **) iter.currentKey();
             uint32 second = *(int32 *) iter.currentValue();
-            int32 start = endianInt32( (uint32) first->asRange()->rangeStart()->asInteger() );
-            int32 end = endianInt32( (uint32) first->asRange()->rangeEnd()->asInteger() );
+            int32 start = (uint32) first->asRange()->rangeStart()->asInteger();
+            int32 end = (uint32) first->asRange()->rangeEnd()->asInteger();
             m_outTemp->write( &start, sizeof( start ) );
             m_outTemp->write( &end, sizeof( end ) );
             tag.addQuerySwitchBlock( second, 0 );
@@ -925,7 +852,7 @@ void GenCode::gen_statement( const Statement *stmt )
          {
             Value *first = *(Value **) iter.currentKey();
             uint32 second = *(int32 *) iter.currentValue();
-            int32 strid = endianInt32( first->asString()->id() );
+            int32 strid = first->asString()->id();
             m_outTemp->write( &strid, sizeof( strid ) );
             tag.addQuerySwitchBlock( second, 0 );
             m_outTemp->write( &dummy, sizeof( dummy ) );
@@ -941,7 +868,7 @@ void GenCode::gen_statement( const Statement *stmt )
             {
                uint32 second = *(int32 *) iter.currentValue();
                Symbol *sym = val->asSymbol();
-               int32 objid = endianInt32( sym->id() );
+               int32 objid = sym->id();
                m_outTemp->write( &objid, sizeof( objid ) );
                tag.addQuerySwitchBlock( second, 0 );
                m_outTemp->write( &dummy, sizeof( dummy ) );
@@ -1017,6 +944,50 @@ void GenCode::gen_statement( const Statement *stmt )
       }
       break;
 
+      case Statement::t_loop:
+      {
+         const StmtLoop *elem = static_cast<const StmtLoop *>( stmt );
+
+         c_jmptag tag( m_outTemp );
+         m_labels_loop.pushBack( &tag );
+
+         // this is the place for begin
+         tag.defineBegin();
+
+         // if we don't have a check, continue can land at loop begin.
+         if ( elem->condition() == 0 )
+            tag.defineNext();
+
+         gen_block( &elem->children() );
+
+         // generate condition check only if present and if not always true
+         if ( elem->condition() == 0 )
+         {
+            // endless loop
+            gen_pcode( P_JMP, c_param_fixed( tag.addQueryBegin() ) );
+         }
+         else {
+            tag.defineNext();
+
+            if ( ! elem->condition()->isTrue() )
+            {
+               if ( elem->condition()->isSimple() ) {
+                  gen_pcode( P_IFF, c_param_fixed( tag.addQueryBegin() ), elem->condition() );
+               }
+               else {
+                  gen_complex_value( elem->condition() );
+                  gen_pcode( P_IFF, c_param_fixed( tag.addQueryBegin() ), e_parA );
+               }
+            }
+         }
+         // if it's true, terminate immediately
+
+         // end of the loop
+         tag.defineEnd();
+         m_labels_loop.popBack();
+      }
+      break;
+
       case Statement::t_propdef:
       {
          const StmtVarDef *pdef = static_cast<const StmtVarDef *>( stmt );
@@ -1033,87 +1004,37 @@ void GenCode::gen_statement( const Statement *stmt )
       case Statement::t_forin:
       {
          const StmtForin *loop = static_cast<const StmtForin *>( stmt );
-         int neededVars = 0;  // vars pushed by reference for target expansion
 
          // begin a new loop
          c_jmptag tag( m_outTemp );
-         tag.setForIn( true );
+         tag.setForIn( loop );
          m_labels_loop.pushBack( &tag );
 
-         // ================================== Trav block
-         // PSHR  $var1
-         // ...
-         // PSHR  $varN
-         // TRAV Jump On Failure, NumberOfVars (immediate int), SOURCE (or A),
-         //            ----or----
-         // TRAV Jump On Failure, $target, SOURCE (or A)
-         //    ---> Push immediate int (counter) or object (iterator)
-         //    ---> Push PSHR $target / immediate integer (num of vars)
-         //    ---> Push source
+         uint32 neededVars = loop->dest()->size();
 
-         //    [FORFIRST Block]
-         //   ...
-         // begin:
-         //    [loop body]
-         //    [If we have a forlast block  --
-         //      TRAL (last)]
-         // [FORMIDDLE block]
-         //
-         // TRAN begin
-         // last:
-         //   [last block]
-         // exit:
-         // ipop 3 + pushed vars.
-
-         if ( loop->dest()->isSimple() ) {
-            if ( loop->source()->isSimple() ) {
-               gen_pcode( P_TRAV, c_param_fixed( tag.addQueryPostEnd() ), loop->dest(), loop->source() );
-            }
-            else {
-               gen_value( loop->source() );
-               gen_pcode( P_TRAV, c_param_fixed( tag.addQueryPostEnd() ), loop->dest(), e_parA );
-            }
+         if( loop->source()->isSimple() )
+         {
+            gen_pcode( P_TRAV,
+                  c_param_fixed( tag.addQueryPostEnd() ),
+                  c_param_fixed( neededVars ),
+                  loop->source() );
          }
-         else {
-            if( loop->dest()->type() == Value::t_array_decl ) {
-               // push in the stack needed vars.
-               ListElement *it_t = loop->dest()->asArray()->begin();
-               neededVars = 0;
-
-               while( it_t != 0 ) {
-                  const Value *val = (const Value *) it_t->data();
-                  if( val->isSimple() )
-                     gen_pcode( P_PSHR, val );
-                  else {
-                     gen_value( val );
-                     gen_pcode( P_PSHR, e_parA );
-                  }
-                  ++neededVars;
-                  it_t = it_t->next();
-               }
-
-               if ( loop->source()->isSimple() ) {
-                  gen_pcode( P_TRAV, c_param_fixed( tag.addQueryPostEnd() ),
-                     (int32) neededVars, loop->source() );
-               }
-               else {
-                  gen_value( loop->source() );
-                  gen_pcode( P_TRAV, c_param_fixed( tag.addQueryPostEnd() ),
-                     (int32) neededVars, e_parA );
-               }
-            }
-            else {
-               gen_value( loop->dest() );
-               if ( loop->source()->isSimple() ) {
-                  gen_pcode( P_TRAV, c_param_fixed( tag.addQueryPostEnd() ), e_parA, loop->source() );
-               }
-               else {
-                  gen_pcode( P_PUSH, e_parA );
-                  gen_value( loop->source() );
-                  gen_pcode( P_POP, e_parB );
-                  gen_pcode( P_TRAV, c_param_fixed( tag.addQueryPostEnd() ), e_parB, e_parA );
-               }
-            }
+         else
+         {
+            gen_value( loop->source() );
+            gen_pcode( P_TRAV,
+                  c_param_fixed( tag.addQueryPostEnd() ),
+                  c_param_fixed( neededVars ),
+                  e_parA );
+         }
+            // generate all the NOPs
+         ListElement* ite = loop->dest()->begin();
+         while( ite != 0 )
+         {
+            Value* v = (Value*) ite->data();
+            fassert( v->isSimple() );
+            gen_pcode( P_NOP, v );
+            ite = ite->next();
          }
 
          // have we got a "first" block?
@@ -1138,20 +1059,29 @@ void GenCode::gen_statement( const Statement *stmt )
          // after the formiddle block we have the next element pick
          tag.defineNext();
 
-         // second parameter 0 --> do not delete.
-         gen_pcode( P_TRAN, c_param_fixed( tag.addQueryBegin() ),
-               c_param_fixed( tag.addQueryElif( 0, 2 ) ),
-               c_param_fixed( 0 ) );
+         gen_pcode( P_TRAN,
+               c_param_fixed( tag.addQueryBegin() ),
+               c_param_fixed( neededVars )
+               );
+         ite = loop->dest()->begin();
+         while( ite != 0 )
+         {
+            Value* v = (Value*) ite->data();
+            fassert( v->isSimple() );
+            gen_pcode( P_NOP, v );
+            ite = ite->next();
+         }
 
          // and the last time...
          tag.defineElif( 0 );
          if( ! loop->lastBlock().empty() ) {
+            tag.setForLast( true );
             gen_block( &loop->lastBlock() );
          }
 
          // create break landing code:
          tag.defineEnd();
-         gen_pcode( P_IPOP, c_param_fixed( 3 + neededVars) );
+         gen_pcode( P_IPOP, c_param_fixed(1) );
          tag.definePostEnd();
 
          m_labels_loop.popBack();
@@ -1195,7 +1125,7 @@ void GenCode::gen_statement( const Statement *stmt )
          // the landing is here
          uint32 curpos = (uint32) m_outTemp->tell();
          m_outTemp->seekBegin( tryRefPos );
-         tryRefPos = endianInt32( curpos );
+         tryRefPos = curpos;
          m_outTemp->write( reinterpret_cast< const char *>(&tryRefPos), sizeof( tryRefPos ) );
          m_outTemp->seekBegin( curpos );
 
@@ -1231,7 +1161,7 @@ void GenCode::gen_statement( const Statement *stmt )
                {
                   Value *first = *(Value **) iter.currentKey();
                   uint32 second = *(int32 *) iter.currentValue();
-                  int64 value = endianInt64( first->asInteger() );
+                  int64 value = first->asInteger();
                   m_outTemp->write( &value, sizeof(value ) );
                   tag.addQuerySwitchBlock( second, 0 );
                   m_outTemp->write( &dummy, sizeof( dummy ) );
@@ -1247,7 +1177,7 @@ void GenCode::gen_statement( const Statement *stmt )
                   {
                      uint32 second = *(int32 *) iter.currentValue();
                      Symbol *sym = val->asSymbol();
-                     int32 objid = endianInt32( sym->id() );
+                     int32 objid = sym->id();
                      m_outTemp->write( &objid, sizeof( objid ) );
                      tag.addQuerySwitchBlock( second, 0 );
                      m_outTemp->write( &dummy, sizeof( dummy ) );
@@ -1304,7 +1234,7 @@ void GenCode::gen_statement( const Statement *stmt )
             // tell the TRY jumper where to jump when we are out of TRY
             curpos = (uint32) m_outTemp->tell();
             m_outTemp->seekBegin( tryEndRefPos );
-            tryEndRefPos = endianInt32( curpos );
+            tryEndRefPos = curpos;
             m_outTemp->write( reinterpret_cast< const char *>(&tryEndRefPos), sizeof( tryEndRefPos ) );
             m_outTemp->seekBegin( curpos );
          }
@@ -1616,8 +1546,6 @@ void GenCode::gen_expression( const Expression *exp, t_valType &xValue )
       case Expression::t_eq: mode = 2; opname = P_EQ; break;
       case Expression::t_neq: mode = 2; opname = P_NEQ; break;
 
-      case Expression::t_has: mode = 2; opname = P_HAS; break;
-      case Expression::t_hasnt: mode = 2; opname = P_HASN; break;
       case Expression::t_in: mode = 2; opname = P_IN; break;
       case Expression::t_notin: mode = 2; opname = P_NOIN; break;
       case Expression::t_provides: mode = 2; opname = P_PROV; break;
@@ -1764,24 +1692,43 @@ void GenCode::gen_expression( const Expression *exp, t_valType &xValue )
 
       case Expression::t_lambda:
          xValue = l_value;
-         gen_pcode( P_STO, e_parA, exp->first()->asSymbol() );
+         if( exp->second() != 0 )
+         {
+            // we must create the lambda closure
+            int size = 0;
+            ListElement *iter = exp->second()->asArray()->begin();
+            while( iter != 0 )
+            {
+               const Value *val = (Value *) iter->data();
+               // push the reference; we want a reference in the closure.
+               gen_pcode( P_PSHR, val );
+               size++;
+               iter = iter->next();
+            }
+
+            gen_pcode( P_CLOS, c_param_fixed(size), e_parA, exp->first()->asSymbol() );
+         }
+         else
+         {
+            gen_pcode( P_STO, e_parA, exp->first()->asSymbol() );
+         }
       return;
-      
+
       case Expression::t_oob:
          xValue = l_value;
          gen_pcode( P_OOB, c_param_fixed(1), exp->first() );
       return;
-      
+
       case Expression::t_deoob:
          xValue = l_value;
          gen_pcode( P_OOB, c_param_fixed(0), exp->first() );
       return;
-      
+
       case Expression::t_xoroob:
          xValue = l_value;
          gen_pcode( P_OOB, c_param_fixed(2), exp->first() );
       return;
-      
+
       case Expression::t_isoob:
          xValue = l_value;
          gen_pcode( P_OOB, c_param_fixed(3), exp->first() );
@@ -1964,11 +1911,14 @@ void GenCode::gen_load( const Value *target, const Value *source )
             gen_store_to_deep( P_STP, exp->first(), exp->second(), source );
          }
       }
-      else if ( target->type() == Value::t_array_decl ) {
+      else if ( target->type() == Value::t_array_decl )
+      {
+         const ArrayDecl *tarr = target->asArray();
+
          // if the source is also an array, fine, we have a 1:1 assignment.
-         if ( source->type() == Value::t_array_decl ) {
+         if ( source->type() == Value::t_array_decl )
+         {
             const ArrayDecl *sarr = source->asArray();
-            const ArrayDecl *tarr = target->asArray();
             ListElement *it_s = sarr->begin();
             ListElement *it_t = tarr->begin();
 
@@ -1987,21 +1937,23 @@ void GenCode::gen_load( const Value *target, const Value *source )
          else {
             // then unpack the source in the array.
             if ( source->isSimple() ) {
-					//gen_refArray( target->asArray() );
-               //gen_pcode( P_UNPK, e_parA, source );
-					int size = gen_refArray( target->asArray(), false );
-					gen_pcode( P_UNPS, c_param_fixed( size ), source );
+               gen_pcode( P_LDAS, c_param_fixed( tarr->size() ), source );
             }
             else {
-               /*gen_complex_value( source );
-               gen_pcode( P_PUSH, e_parA );
-					gen_refArray( target->asArray() );
-               gen_pcode( P_POP, e_parB );
-               gen_pcode( P_UNPK, e_parA, e_parB );
-					*/
 					gen_complex_value( source );
-					int size = gen_refArray( target->asArray(), false );
-               gen_pcode( P_UNPS, c_param_fixed( size ), e_parA );
+					gen_pcode( P_LDAS, c_param_fixed( tarr->size() ), e_parA );
+            }
+
+            ListElement *it_t = tarr->end();
+            while( it_t != 0 ) {
+               const Value *t = (const Value *) it_t->data();
+               if( t->isSimple() )
+                   gen_pcode( P_POP, t );
+               else {
+                  gen_pcode( P_POP, e_parB );
+                  gen_load_from_reg( t, e_parB );
+               }
+               it_t = it_t->prev();
             }
          }
       }
@@ -2152,30 +2104,37 @@ void GenCode::gen_load_from_reg( const Value *target, t_paramType reg )
          else if ( exp->type() == Expression::t_obj_access ) {
             gen_store_to_deep_reg( P_STP, exp->first(), exp->second(), reg );
          }
+         else {
+            gen_pcode( P_PUSH, reg );
+            gen_expression( exp );
+            gen_pcode( P_POP, e_parB );
+            gen_pcode( P_LD, e_parA, e_parB );
+         }
       }
-      else if ( target->type() == Value::t_array_decl ) {
+      else if ( target->type() == Value::t_array_decl )
+      {
          // if the source is also an array, fine, we have a 1:1 assignment.
          const ArrayDecl *tarr = target->asArray();
-         ListElement *it_t = tarr->begin();
-         int size = 0;
+         // push the source array on the stack.
+         gen_pcode( P_LDAS, c_param_fixed( tarr->size() ), reg );
 
-         gen_pcode( P_PUSH, reg );
+         // and load each element back in the array
+         ListElement *it_t = tarr->end();
 
-         // first generates an array of references
-         while( it_t != 0 ) {
-            // again, is the compiler that must make sure of this...
+         // Now load each element by popping it.
+         while( it_t != 0 )
+         {
             const Value *val = (const Value *) it_t->data();
-            fassert( val->isSimple() );
-
-            gen_pcode( P_PSHR, val );
-
-            ++size;
-            it_t = it_t->next();
+            if( val->isSimple() )
+            {
+               gen_pcode( P_POP, val );
+            }
+            else {
+               gen_pcode( P_POP, e_parB );
+               gen_load_from_reg( val, e_parB );
+            }
+            it_t = it_t->prev();
          }
-
-         gen_pcode( P_GENA, c_param_fixed( size ) );
-         gen_pcode( P_POP, e_parB );
-         gen_pcode( P_UNPK, e_parA, e_parB );
       }
    }
 }
@@ -2256,7 +2215,7 @@ void GenCode::gen_funcall( const Expression *exp, bool fork )
       // landing for fork
       uint32 curpos = (uint32) m_outTemp->tell();
       m_outTemp->seekBegin( fork_pos );
-      fork_pos = endianInt32( curpos );
+      fork_pos = curpos;
       m_outTemp->write( &fork_pos, sizeof( fork_pos ) );
       m_outTemp->seekBegin( curpos );
    }
