@@ -22,9 +22,9 @@
 #include <falcon/setup.h>
 #include <falcon/item.h>
 #include <falcon/vm.h>
+#include <falcon/corefunc.h>
 #include <falcon/stream.h>
 #include <falcon/lineardict.h>
-#include <falcon/attribute.h>
 #include <falcon/membuf.h>
 
 namespace Falcon {
@@ -45,41 +45,56 @@ bool Item::serialize_symbol( Stream *file, const Symbol *sym ) const
 }
 
 
-bool Item::serialize_function( Stream *file, const Symbol *func ) const
+bool Item::serialize_function( Stream *file, const CoreFunc *func, bool bLive ) const
 {
    byte type = FLC_ITEM_FUNC;
+   if ( bLive )
+         type |= 0x80;
    file->write( &type, 1 );
 
    // we don't serialize the module ID because it is better to rebuild it on deserialization
-   serialize_symbol( file, func );
+   serialize_symbol( file, func->symbol() );
 
-   if( func->isFunction() )
+   if ( func->symbol()->isFunction() )
    {
-      FuncDef *fdef = func->getFuncDef();
+      // language function ? -- serialize the state.
+      FuncDef *fdef = func->symbol()->getFuncDef();
       // write the called status
       uint32 itemId = fdef->onceItemId();
       if ( itemId != FuncDef::NO_STATE )
       {
-         byte called = asModule()->globals().itemAt( itemId ).isNil() ? 0 : 1;
+         byte called = func->liveModule()->globals()[ itemId ].isNil() ? 0 : 1;
          file->write( &called, 1 );
       }
 
-      if ( ! file->good() )
-         return false;
+      // and we may need to serialize also it's closure.
+      ItemArray* closed = func->closure();
+      if( closed != 0 )
+      {
+         int32 len = endianInt32( closed->length() );
+         file->write( (byte *) &len, sizeof( len ) );
+
+         for( uint32 i = 0; i < closed->length(); i ++ )
+         {
+            (*closed)[i].serialize( file, bLive );
+            if( ! file->good() )
+               return false;
+         }
+      }
+      else {
+         int32 len = 0;
+         file->write( (byte *) &len, sizeof( len ) );
+      }
    }
 
+   if ( ! file->good() )
+      return false;
+
    return true;
 }
 
 bool Item::serialize_object( Stream *file, CoreObject *obj, bool bLive ) const
 {
-   // if we're not live, we can't have user data
-   if ( ! bLive && obj->getUserData() != 0 )
-   {
-      obj->origin()->raiseError( e_unserializable, "Item::serialize" );
-      return false;
-   }
-
    byte type = FLC_ITEM_OBJECT;
    if ( bLive )
       type |= 0x80;
@@ -87,68 +102,10 @@ bool Item::serialize_object( Stream *file, CoreObject *obj, bool bLive ) const
    file->write( &type, 1 );
 
    // write the class symbol so that it can be rebuilt back.
-   serialize_symbol( file, obj->instanceOf() );
+   serialize_symbol( file, obj->generator()->symbol() );
 
-   // then serialzie the properties
-   uint32 len = endianInt32( obj->propCount() );
-   file->write((byte *) &len, sizeof(len) );
-
-   for( uint32 i = 0; i < len; i ++ ) {
-      Item temp;
-      obj->getPropertyAt( i, temp );
-      if ( temp.serialize( file, bLive ) != sc_ok )
-         return false;
-   }
-
-   // and the attributes
-   AttribHandler *head = obj->attributes();
-   uint32 att_count = 0;
-   while( head != 0 )
-   {
-      att_count++;
-      head = head->next();
-   }
-
-   // write the size...
-   len = endianInt32( att_count );
-   file->write((byte *) &len, sizeof(len) );
-
-   // and then serialize...
-   head = obj->attributes();
-   while( head != 0 )
-   {
-      head->attrib()->name().serialize( file );
-      if ( ! file->good() )
-         return false;
-
-      head = head->next();
-   }
-
-   // finally, clone if live
-   if ( bLive )
-   {
-      if ( obj->getUserData() != 0 )
-      {
-         void *cloned = obj->getObjectManager()->onClone( obj->origin(), obj->getUserData() );
-         if ( cloned == 0 )
-         {
-            obj->origin()->raiseError( e_uncloneable, "Item::serialize" );
-            return false;
-         }
-         file->write( (byte *) &cloned, sizeof( cloned ) );
-      }
-      else {
-         void *data = 0;
-         file->write( (byte *) &data, sizeof( data ) );
-      }
-
-      if ( ! file->good() )
-      {
-         return false;
-      }
-   }
-
-   return true;
+   // then serialzie the object itself
+   return obj->serialize( file, bLive );
 }
 
 
@@ -205,15 +162,15 @@ Item::e_sercode Item::serialize( Stream *file, bool bLive ) const
          byte type = FLC_ITEM_RANGE;
          file->write((byte *) &type, 1 );
 
-         int32 val1 = endianInt32(this->asRangeStart());
-         int32 val2 = endianInt32(this->asRangeEnd());
-         int32 val3 = endianInt32(this->asRangeStep());
-         byte isOpen = this->asRangeIsOpen() ? 1 : 0;
+         int64 val1 = endianInt64(this->asRangeStart());
+         int64 val2 = endianInt64(this->asRangeEnd());
+         int64 val3 = endianInt64(this->asRangeStep());
+         //byte isOpen = this->asRangeIsOpen() ? 1 : 0;
 
          file->write( (byte *) &val1, sizeof( val1 ) );
          file->write( (byte *) &val2, sizeof( val2 ) );
          file->write( (byte *) &val3, sizeof( val3 ) );
-         file->write( (byte *) &isOpen, sizeof( isOpen ) );
+         //file->write( (byte *) &isOpen, sizeof( isOpen ) );
       }
       break;
 
@@ -227,15 +184,6 @@ Item::e_sercode Item::serialize( Stream *file, bool bLive ) const
       }
       break;
 
-      case FLC_ITEM_ATTRIBUTE:
-      {
-         byte type = FLC_ITEM_ATTRIBUTE;
-         file->write( &type, 1 );
-
-         this->asAttribute()->name().serialize( file );
-      }
-      break;
-
       case FLC_ITEM_STRING:
       {
          byte type = FLC_ITEM_STRING;
@@ -258,24 +206,16 @@ Item::e_sercode Item::serialize( Stream *file, bool bLive ) const
       {
          byte type = FLC_ITEM_MEMBUF;
          if ( bLive )
+         {
             type |= 0x80;
-
-         file->write( &type, 1 );
-         this->asMemBuf()->serialize( file, bLive );
-      }
-      break;
-
-      case FLC_ITEM_FBOM:
-      {
-         byte type = FLC_ITEM_FBOM;
-         file->write((byte *) &type, 1 );
-
-         type = this->getFbomMethod();
-         file->write((byte *) &type, 1 );
-
-         Item ifbom;
-         getFbomItem( ifbom );
-         ifbom.serialize( file, bLive );
+            file->write( &type, 1 );
+            MemBuf* mb = asMemBuf();
+            file->write( &mb, sizeof(mb) );
+         }
+         else {
+            file->write( &type, 1 );
+            this->asMemBuf()->serialize( file, bLive );
+         }
       }
       break;
 
@@ -307,56 +247,43 @@ Item::e_sercode Item::serialize( Stream *file, bool bLive ) const
          int32 len = endianInt32( dict->length() );
          file->write( (byte *) &len, sizeof( len ) );
 
-         Item key, value;
-         dict->traverseBegin();
-         while( dict->traverseNext( key, value ) )
+         Iterator iter( &dict->items() );
+         while( iter.hasCurrent() )
          {
-            key.serialize( file, bLive );
+            iter.getCurrentKey().serialize( file, bLive );
             if( ! file->good() )
                return sc_ferror;
-            value.serialize( file, bLive );
+            iter.getCurrent().serialize( file, bLive );
             if( ! file->good() )
                return sc_ferror;
+
+            iter.next();
          }
       }
       break;
 
       case FLC_ITEM_FUNC:
-         serialize_function( file, this->asFunction() );
+         serialize_function( file, this->asFunction(), bLive );
       break;
 
       case FLC_ITEM_METHOD:
       {
          byte type = FLC_ITEM_METHOD;
-         if ( bLive )
-         {
-            type |= 0x80;
-            file->write( &type, 1 );
-            void *obj = this->asMethodObject();
-            file->write( &obj, sizeof( obj ) );
-         }
-         else {
-            // TODO
-            serialize_object( file, this->asMethodObject(), bLive );
-         }
-         serialize_function( file, this->asMethodFunction() );
-      }
-      break;
-
-      case FLC_ITEM_TABMETHOD:
-      {
-         byte type = FLC_ITEM_TABMETHOD;
          file->write( &type, 1 );
-         serialize_function( file, this->asMethodFunction() );
 
-         if( isTabMethodDict() )
+         e_sercode sc = asMethodItem().serialize( file, bLive );
+         if( sc != sc_ok )
+            return sc;
+
+         CallPoint* cp = this->asMethodFunc();
+         if ( cp->isFunc() )
          {
-            Item temp = asTabMethodDict();
-            temp.serialize( file, bLive );
+            serialize_function( file, static_cast<CoreFunc*>(cp), bLive );
          }
-         else {
-            Item temp = asTabMethodArray();
-            temp.serialize( file, bLive );
+         else
+         {
+            SafeItem arr( static_cast<CoreArray*>(cp) );
+            arr.serialize(file, bLive );
          }
       }
       break;
@@ -375,7 +302,7 @@ Item::e_sercode Item::serialize( Stream *file, bool bLive ) const
 
        case FLC_ITEM_CLSMETHOD:
          serialize_class( file, this->asMethodClass() );
-         serialize_function( file, this->asMethodFunction() );
+         serialize_function( file, this->asFunction(), bLive );
       break;
 
       default:
@@ -439,7 +366,7 @@ Item::e_sercode Item::deserialize_function( Stream *file, VMachine *vm )
 
    if ( ! sym->isFunction() ) {
       // external function
-      setFunction( sym, lmod );
+      setFunction( new CoreFunc( sym, lmod ) );
       return sc_ok;
    }
 
@@ -453,12 +380,33 @@ Item::e_sercode Item::deserialize_function( Stream *file, VMachine *vm )
       byte called;
       file->read( &called, 1 );
       if( called )
-         lmod->globals().itemAt( itemId ).setInteger( 1 );
+         lmod->globals()[ itemId ].setInteger( 1 );
       else
-         lmod->globals().itemAt( itemId ).setNil();
+         lmod->globals()[ itemId ].setNil();
    }
 
-   setFunction( sym, lmod );
+   CoreFunc* function = new CoreFunc( sym, lmod );
+   // read the closure state.
+   uint32 closlen;
+   file->read( &closlen, sizeof( closlen ) );
+   if( closlen != 0 )
+   {
+      ItemArray* closure = new ItemArray( closlen );
+      closure->resize( closlen );
+
+      // put the state in now, so we can destroy it in case of error.
+      function->closure( closure );
+
+      // deserialize all the items.
+      for( uint32 i = 0; i < closlen; i++ ) {
+         Item::e_sercode res = (*closure)[i].deserialize(file, vm );
+         if ( res != sc_ok )
+            return res;
+      }
+
+   }
+
+   setFunction( function );
    return sc_ok;
 }
 
@@ -497,8 +445,9 @@ Item::e_sercode Item::deserialize_class( Stream *file, VMachine *vm )
 
 Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
 {
-   byte type;
-   file->read((byte *) &type, 1 );
+   byte type = FLC_ITEM_NIL;
+   if ( file->read((byte *) &type, 1 ) == 0 )
+      return sc_eof;
 
    if( ! file->good() )
       return sc_ferror;
@@ -535,18 +484,22 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
 
       case FLC_ITEM_RANGE:
       {
-         int32 val1;
-         int32 val2;
-         int32 val3;
-         byte isOpen;
+         int64 val1;
+         int64 val2;
+         int64 val3;
+         //byte isOpen;
 
          file->read( (byte *) &val1, sizeof( val1 ) );
          file->read( (byte *) &val2, sizeof( val2 ) );
          file->read( (byte *) &val3, sizeof( val3 ) );
-         file->read( (byte *) &isOpen, sizeof( isOpen ) );
-         
+         //file->read( (byte *) &isOpen, sizeof( isOpen ) );
+
+         val1 = endianInt64( val1 );
+         val2 = endianInt64( val2 );
+         val3 = endianInt64( val3 );
+
          if ( file->good() ) {
-            setRange( val1, val2, val3, isOpen != 0 );
+            setRange( new CoreRange( val1, val2, val3 ) );
             return sc_ok;
          }
          return sc_ferror;
@@ -565,23 +518,6 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
       }
       break;
 
-      case FLC_ITEM_ATTRIBUTE:
-      {
-         String aname;
-
-         if ( ! aname.deserialize( file ) )
-         {
-            return file->bad() ? sc_ferror : sc_invformat;
-         }
-
-         Attribute *attrib = vm->findAttribute( aname );
-         if ( attrib == 0 )
-            return sc_missclass;
-
-         setAttribute( attrib );
-      }
-      break;
-
       case FLC_ITEM_LBIND:
       {
          int32 id;
@@ -590,18 +526,17 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          if ( ! name.deserialize( file ) )
             return file->bad() ? sc_ferror : sc_invformat;
 
-         setLBind( new GarbageString( vm, name ) );
+         setLBind( new CoreString( name ) );
       }
       break;
 
       case FLC_ITEM_STRING:
       {
-         GarbageString *cs = new GarbageString( vm );
+         CoreString *cs = new CoreString;
          setString( cs );
 
          if ( ! cs->deserialize( file ) )
          {
-            delete cs;
             return file->bad() ? sc_ferror : sc_invformat;
          }
 
@@ -616,7 +551,7 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
       case FLC_ITEM_MEMBUF |0x80:
       {
          // get the function pointer in the stream
-         MemBuf *(*deserializer)( VMachine *, Stream * );
+         /*MemBuf *(*deserializer)( VMachine *, Stream * );
          file->read( &deserializer, sizeof( deserializer ) );
          if ( ! file->good() ) {
             return sc_ferror;
@@ -626,10 +561,15 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          if( mb == 0 )
          {
             return sc_invformat;
-         }
+         }*/
 
-         setMemBuf( mb );
-         return sc_ok;
+         MemBuf* mb;
+         if( file->read( &mb, sizeof( mb ) ) == sizeof(mb) )
+         {
+            setMemBuf( mb );
+            return sc_ok;
+         }
+         return sc_eof;
       }
 
       case FLC_ITEM_MEMBUF:
@@ -640,27 +580,11 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
             return sc_ok;
          }
 
-         delete mb; // may be 0, but it's ok
          return sc_ferror;
       }
       break;
 
 
-      case FLC_ITEM_FBOM:
-      {
-         byte methodId;
-         //TODO: Verify if methodId is in a valid range.
-         file->read( &methodId, 1 );
-
-         Item bommed;
-         e_sercode code = bommed.deserialize( file, vm );
-         if ( code != sc_ok )
-            return code;
-
-         setFbom( bommed, methodId );
-         return sc_ok;
-      }
-
       case FLC_ITEM_ARRAY:
       {
          if( vm == 0 )
@@ -673,11 +597,11 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          if ( file->good() )
          {
             val = endianInt32(val);
-            CoreArray *array = new CoreArray( vm, val );
+            CoreArray *array = new CoreArray( val );
 
             for( int i = 0; i < val; i ++ )
             {
-               retval = array->elements()[i].deserialize( file, vm );
+               retval = array->items()[i].deserialize( file, vm );
                if( retval != sc_ok ) {
                   break;
                }
@@ -708,7 +632,7 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          if ( file->good() )
          {
             val = endianInt32(val);
-            LinearDict *dict = new LinearDict( vm, val );
+            LinearDict *dict = new LinearDict( val );
             LinearDictEntry *elems = dict->entries();
             e_sercode retval = sc_ok;
             for( int i = 0; i < val; i ++ ) {
@@ -723,27 +647,30 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
             }
 
             if( retval == sc_ok ) {
-               dict->bless( blessed ? true : false );
-               setDict( dict );
+               CoreDict* cdict = new CoreDict( dict );
+               cdict->bless( blessed ? true : false );
+               setDict( cdict );
 
                return sc_ok;
             }
+            else
+               delete dict;
 
-            delete  dict;
             return retval;
          }
       }
       break;
 
-
+      case FLC_ITEM_FUNC | 0x80:
       case FLC_ITEM_FUNC:
+      {
          if( vm == 0 )
             return sc_missvm;
 
          return deserialize_function( file, vm );
-      break;
+      }
+     break;
 
-      case FLC_ITEM_METHOD | 0x80:
       case FLC_ITEM_METHOD:
       {
          if( vm == 0 )
@@ -752,59 +679,26 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          Item obj;
          Item func;
          e_sercode sc;
-         if( type == FLC_ITEM_METHOD )
+         sc = obj.deserialize( file, vm );
+         if ( sc != sc_ok )
+            return sc;
+
+         sc = func.deserialize( file, vm );
+         if ( sc != sc_ok )
+            return sc;
+         if ( func.isFunction() )
+            setMethod( obj, func.asMethodFunc() );
+         else if ( func.isArray() && func.isCallable() )
          {
-            // TODO
-            sc = obj.deserialize( file, vm );
-            if ( sc != sc_ok )
-               return sc;
-            if ( ! obj.isObject() )
-               return sc_invformat;
+            setMethod( obj, func.asArray() );
          }
-         else {
-            CoreObject *memObj;
-            file->read( &memObj, sizeof( memObj ) );
-            obj = memObj;
-         }
-
-         sc = func.deserialize( file, vm );
-         if ( sc != sc_ok )
-            return sc;
-         if ( ! func.isFunction() )
-            return sc_invformat;
-
-         setMethod( obj.asObjectSafe(), func.asFunction(), func.asModule() );
-         return sc_ok;
-      }
-
-      case FLC_ITEM_TABMETHOD:
-      {
-         if( vm == 0 )
-            return sc_missvm;
-
-         Item vector;
-         Item func;
-         e_sercode sc;
-
-         sc = func.deserialize( file, vm );
-         if ( sc != sc_ok )
-            return sc;
-         if ( ! func.isFunction() )
-            return sc_invformat;
-
-         sc = vector.deserialize( file, vm );
-         if ( sc != sc_ok )
-            return sc;
-         if ( vector.isArray() )
-            setTabMethod( vector.asArray(), func.asFunction(), func.asModule() );
-         else if ( vector.isDict() )
-            setTabMethod( vector.asArray(), func.asFunction(), func.asModule() );
          else
             return sc_invformat;
 
          return sc_ok;
       }
 
+
       case FLC_ITEM_OBJECT | 0x80:
       case FLC_ITEM_OBJECT:
       {
@@ -820,70 +714,17 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
          if ( sc != sc_ok  )
             return sc;
 
-         Item *clitem = lmod->globals().itemPtrAt( sym->itemId() );
+         Item *clitem = &lmod->globals()[ sym->itemId() ];
 
          // Create the core object, but don't fill attribs.
-         CoreObject *object = clitem->asClass()->createInstance(0, false);
-
-         // Read the class property table.
-         uint32 len;
-         file->read( (byte *) &len, sizeof( len ) );
-         len = endianInt32( len );
-         if ( len != object->propCount() ){
+         CoreObject *object = clitem->asClass()->createInstance(0, true);
+         if ( ! object->deserialize( file, bLive ) )
+         {
             return sc_missclass;
          }
 
-         // sc must be sc_ok
-         for( uint32 i = 0; i < len; i ++ ) {
-            if( ! file->good() ) {
-               sc = sc_ferror;
-               break;
-            }
-
-            Item temp;
-            sc = temp.deserialize( file, vm );
-            object->setPropertyAt( i, temp );
-            if ( sc != sc_ok ) {
-               return sc;
-            }
-         }
-
-         // now deserialize the attributes
-         file->read( (byte *) &len, sizeof( len ) );
-         len = endianInt32( len );
-         for( uint32 j = 0; j < len; j ++ )
-         {
-            // first the string, which is the name...
-            String a_name;
-            if( ! a_name.deserialize( file ) ) {
-               return file->good() ? sc_invformat : sc_ferror;
-            }
-
-            // then the attribute
-            Attribute *attrib = vm->findAttribute( a_name );
-            if( attrib == 0 )
-            {
-               return sc_missclass;
-            }
-
-            attrib->giveTo( object );
-         }
-
-         if ( sc == sc_ok ) {
-            // if the object was serialized live, get the user data.
-            if( bLive )
-            {
-               void *data;
-               if ( file->read( (byte *) &data, sizeof( data ) ) != sizeof( data ) )
-                  return sc_ferror;
-               object->setUserData( data );
-            }
-
-            setObject( object );
-            return sc_ok;
-         }
-
-         return sc;
+         setObject( object );
+         return file->good() ? sc_ok : sc_ferror;
       }
       break;
 
@@ -909,14 +750,14 @@ Item::e_sercode Item::deserialize( Stream *file, VMachine *vm )
 
 
 //TODO:Move in another file.
-bool Item::clone( Item &target, VMachine *vm ) const
+bool Item::clone( Item &target) const
 {
    const Item *item = this->dereference();
 
    switch( item->type() )
    {
       case FLC_ITEM_STRING:
-         target = new GarbageString( vm, *item->asString() );
+         target = new CoreString( *item->asString() );
       break;
 
       case FLC_ITEM_ARRAY:
@@ -939,27 +780,7 @@ bool Item::clone( Item &target, VMachine *vm ) const
 
       case FLC_ITEM_METHOD:
       {
-         CoreObject *clone = item->asMethodObject()->clone();
-         if ( clone == 0 ) {
-            return false;
-         }
-         target.setMethod( clone, item->asMethodFunction(), item->asModule() );
-      }
-      break;
-
-      case FLC_ITEM_TABMETHOD:
-      {
-         if( item->isTabMethodDict() )
-         {
-            CoreDict *clone = item->asTabMethodDict()->clone();
-            if ( clone == 0 ) return false;
-            target.setTabMethod( clone, item->asMethodFunction(), item->asModule() );
-         }
-         else {
-            CoreArray *clone = item->asTabMethodArray()->clone();
-            if ( clone == 0 ) return false;
-            target.setTabMethod( clone, item->asMethodFunction(), item->asModule() );
-         }
+         target = *this;
       }
       break;
 
@@ -970,6 +791,33 @@ bool Item::clone( Item &target, VMachine *vm ) const
 
    return true;
 }
+/*
+void Item::copy( const Item &other )
+{
+   if ( isLBind() )
+   {
+      VMachine* current = VMachine::getCurrent();
+      if ( current != 0 )
+      {
+         current->bindItem( *asLBind(), other );
+         return;
+      }
+
+   }
+
+   if( other.isLBind() )
+   {
+      VMachine* current = VMachine::getCurrent();
+      if ( current != 0 )
+      {
+         current->unbindItem( *other.asLBind(), *this );
+         return;
+      }
+   }
+
+   all = other.all;
+}
+*/
 
 }
 
