@@ -43,9 +43,6 @@ SrcLexer::SrcLexer( Compiler *comp ):
    m_line( 1 ),
    m_previousLine( 1 ),
    m_character( 0 ),
-   m_contexts(0),
-   m_squareContexts(0),
-   m_ctxOpenLine(0),
    m_prevStat(0),
    m_firstEq( false ),
    m_done( false ),
@@ -54,13 +51,15 @@ SrcLexer::SrcLexer( Compiler *comp ):
    m_bIsDirectiveLine(false),
    m_incremental( false ),
    m_lineContContext( false ),
+   m_graphAgain( false ),
    m_chrEndString(0),
    m_in( 0 ),
    m_compiler( comp ),
    m_state( e_line ),
    m_mode( t_mNormal ),
    m_bParsingFtd(false),
-   m_bWasntEmpty(false)
+   m_bWasntEmpty(false),
+   m_topCtx(0)
 {}
 
 SrcLexer::~SrcLexer()
@@ -70,12 +69,12 @@ SrcLexer::~SrcLexer()
 
 void SrcLexer::reset()
 {
+   resetContexts();
+   m_addEol = false; // revert what's done by resetContext
+   
    m_prevStat = 0;
-   m_contexts = 0;
-   m_squareContexts = 0;
    m_character = 0;
    m_state = e_line;
-   m_ctxOpenLine = 0;
    m_bIsDirectiveLine = false;
    m_bWasntEmpty = false;
    m_whiteLead = "";
@@ -209,7 +208,7 @@ int SrcLexer::lex_outscape()
                // and break from the loop so to return the string to print.
                break;
             }
-            else if ( chr == ' ' || chr == '\t' || chr == '\r' || chr == '\n' )
+            else if ( chr == ' ' || chr == '\t' || chr == '\n' )
             {
                // reset lead chars and eventually remove last \n
                m_whiteLead.size(0);
@@ -283,7 +282,7 @@ int SrcLexer::lex_outscape()
          break;
 
          case e_escL:
-            if ( chr == ' ' || chr == '\t' || chr == '\r' || chr == '\n')
+            if ( chr == ' ' || chr == '\t' || chr == '\n')
             {
                // reset lead chars and eventually remove last \n
                m_whiteLead.size(0);
@@ -359,7 +358,7 @@ int SrcLexer::lex_normal()
       if ( ! (m_done && incremental() && hasOpenContexts()) )
       {
          m_lineFilled = false;
-         if ( m_contexts == 0 && m_squareContexts == 0 )
+         if ( ! inParCtx() )
          {
             m_bIsDirectiveLine = false;
             return EOL;
@@ -367,6 +366,12 @@ int SrcLexer::lex_normal()
       }
    }
 
+   if( m_graphAgain )
+   {
+      m_graphAgain = false;
+      return CLOSE_GRAPH;
+   }
+
    if ( m_done )
    {
       // raise error if there is some lexer context open
@@ -450,6 +455,10 @@ int SrcLexer::lex_normal()
 
       m_character++;
 
+      // Totally ignore '\r'
+      if ( chr == '\r' )
+         continue;
+
       switch ( m_state )
       {
          case e_line:
@@ -490,9 +499,22 @@ int SrcLexer::lex_normal()
                   // recognize only the i" for now
                   if ( m_string[0] == 'i' )
                   {
-                     // starting a string context
+                     uint32 nextChr;
+                     // we'll begin to read a string.
+                     if ( readAhead( nextChr ) && nextChr == '\n' )
+                     {
+                        m_mlString = true;
+                        m_line++;
+                        m_character = 0;
+                        m_in->discardReadAhead( 1 );
+                        m_state = chr == '"' ? e_stringRunning : e_litString;
+                     }
+                     else {
+                        m_state = chr == '"' ? e_string : e_litString;
+                     }
+                     
+                     pushContext( ct_string, m_line );
                      m_string.size(0); // remove first char.
-                     m_state = chr == '"' ? e_string : e_litString;;
                      m_string.exported( true );
                      m_chrEndString = chr; //... up to the matching "
                      break;
@@ -570,7 +592,8 @@ int SrcLexer::lex_normal()
                // We have aknowledged this can't be a valid token.
                m_in->unget( chr );
                m_state = e_line;
-               m_compiler->raiseError( e_inv_token, m_string, m_line );
+               m_string = m_string.subString( 0, 1 );
+               m_compiler->raiseError( e_inv_token, "'" + m_string + "'", m_line );
                // do not return.
             }
 
@@ -584,7 +607,6 @@ int SrcLexer::lex_normal()
          }
          break;
 
-         case e_eolCommentString:
          case e_eolComment:
             if ( chr == '\n' )
             {
@@ -592,10 +614,10 @@ int SrcLexer::lex_normal()
                m_line ++;
                m_character = 0;
                m_bIsDirectiveLine = false;
-               m_state = (m_state == e_eolCommentString) ? e_postString: e_line;
+               m_state = e_line;
 
                // a real EOL has been provided here.
-               if ( m_state == e_line && m_contexts == 0  && m_squareContexts == 0 )
+               if ( m_state == e_line && ! inParCtx() )
                {
                   m_bIsDirectiveLine = false;
                   if ( m_lineFilled )
@@ -607,7 +629,6 @@ int SrcLexer::lex_normal()
             }
          break;
 
-         case e_blockCommentString:
          case e_blockComment:
             if ( chr == '\n' )
             { // previous line stays the same
@@ -618,11 +639,11 @@ int SrcLexer::lex_normal()
             else if ( chr == '*' )
             {
                uint32 nextChr;
-               m_in->readAhead( nextChr );
+               readAhead( nextChr );
                if ( nextChr == '/' )
                {
                   m_in->discardReadAhead( nextChr );
-                  m_state = m_state == e_blockCommentString ? e_postString : e_line;
+                  m_state = e_line;
                }
             }
          break;
@@ -633,27 +654,33 @@ int SrcLexer::lex_normal()
             {
                // a method on a number?
                uint32 nextChr;
-               if ( m_in->readAhead( nextChr ) && (nextChr < '0' || nextChr > '9') )
+               if ( readAhead( nextChr ) && (nextChr < '0' || nextChr > '9') )
                {
                   // end
                   m_in->unget( chr );
                   int64 retval;
                   if ( ! m_string.parseInt( retval ) )
                      m_compiler->raiseError( e_inv_num_format, m_line );
-   
+
                   VALUE->integer = retval;
                   m_state = e_line;
                   return INTNUM;
                }
-               
+
                // no.
+               m_string.append( chr );
                m_state = e_floatNumber;
             }
             else if ( chr == 'e' )
             {
                m_state = e_floatNumber_e;
+               m_string.append( chr );
             }
-            else if ( chr < '0' || chr > '9' )
+            else if ( chr >= '0' && chr <= '9' )
+            {
+               m_string.append( chr );
+            }
+            else if ( chr != '_' )
             {
                // end
                m_in->unget( chr );
@@ -665,8 +692,6 @@ int SrcLexer::lex_normal()
                m_state = e_line;
                return INTNUM;
             }
-
-            m_string.append( chr );
          break;
 
          case e_floatNumber:
@@ -676,11 +701,15 @@ int SrcLexer::lex_normal()
                m_state = e_floatNumber_e;
                m_string.append( chr );
             }
-            else if ( chr < '0' || chr > '9'  )
+            else if ( chr >= '0' && chr <= '9'  )
+            {
+               m_string.append( chr );
+            }
+            else if ( chr != '_' )
             {
                // end
                m_in->unget( chr );
-               
+
                // "0." ?
                if( m_string == "0." )
                {
@@ -689,7 +718,7 @@ int SrcLexer::lex_normal()
                   m_state = e_line;
                   return INTNUM;
                }
-               
+
                numeric retval;
                if ( ! m_string.parseDouble( retval ) )
                   m_compiler->raiseError( e_inv_num_format, m_line );
@@ -698,8 +727,6 @@ int SrcLexer::lex_normal()
                m_state = e_line;
                return DBLNUM;
             }
-            else
-               m_string.append( chr );
          break;
 
          case e_floatNumber_e:
@@ -724,7 +751,7 @@ int SrcLexer::lex_normal()
             {
                // end
                m_in->unget( chr );
-               
+
                numeric retval;
                if ( ! m_string.parseDouble( retval ) )
                   m_compiler->raiseError( e_inv_num_format, m_line );
@@ -740,7 +767,20 @@ int SrcLexer::lex_normal()
          case e_zeroNumber:
             m_lineFilled = true;
             if( chr == 'x' || chr == 'X' )
+            {
+               m_string.size( 0 );
                m_state = e_hexNumber;
+            }
+            else if ( chr == 'b' || chr == 'B' )
+            {
+               m_string.size( 0 );
+               m_state = e_binNumber;
+            }
+            else if ( chr == 'c' || chr == 'C' )
+            {
+               m_string.size( 0 );
+               m_state = e_octNumber;
+            }
             else if ( chr >= '0' && chr <= '7' )
             {
                m_string.size( 0 );
@@ -767,7 +807,11 @@ int SrcLexer::lex_normal()
 
          case e_octNumber:
             m_lineFilled = true;
-            if ( chr < '0' || chr > '7' )
+            if ( chr >= '0' && chr <= '7' )
+            {
+               m_string.append( chr );
+            }
+            else if ( chr != '_' )
             {
                m_in->unget( chr );
                uint64 retval;
@@ -778,8 +822,23 @@ int SrcLexer::lex_normal()
                m_state = e_line;
                return INTNUM;
             }
-            else
+         break;
+
+         case e_binNumber:
+            m_lineFilled = true;
+            if ( chr == '0' || chr == '1' )
                m_string.append( chr );
+            else if ( chr != '_' )
+            {
+               m_in->unget( chr );
+               uint64 retval;
+               if ( ! m_string.parseBin( retval ) )
+                  m_compiler->raiseError( e_inv_num_format, m_line );
+
+               VALUE->integer = retval;
+               m_state = e_line;
+               return INTNUM;
+            }
          break;
 
          case e_hexNumber:
@@ -791,7 +850,7 @@ int SrcLexer::lex_normal()
             {
                   m_string.append( chr );
             }
-            else
+            else if ( chr != '_' )
             {
                m_in->unget( chr );
                uint64 retval;
@@ -806,37 +865,37 @@ int SrcLexer::lex_normal()
 
          case e_litString:
             m_lineFilled = true;
-            // an escape ?
-            if ( chr == '\\' )
+            if ( chr == '\n' )
             {
-               uint32 nextChar;
-               m_in->readAhead( nextChar );
-               if( nextChar == '\'' )
+               if( ! m_mlString )
                {
-
-                  m_in->discardReadAhead( 1 );
-                  m_string.append( '\'' );
+                  m_compiler->raiseError( e_nl_in_lit, m_line );
+                  m_lineFilled = false;
+                  m_character = 0;
+                  m_bIsDirectiveLine = false;
+                  m_compiler->raiseError( e_nl_in_lit, m_previousLine );
+                  m_state = e_line;
+                  popContext();
                }
-               else {
-                  m_string.append( '\\' );
-               }
-            }
-            else if ( chr == '\n' )
-            {
+               m_string.append( chr );
                m_previousLine = m_line;
                m_line++;
-               m_lineFilled = false;
-               m_character = 0;
-               m_bIsDirectiveLine = false;
-               m_compiler->raiseError( e_nl_in_lit, m_previousLine );
-               m_state = e_line;
             }
             else if ( chr == '\'' )
             {
-               m_state = e_line;
-               VALUE->stringp = m_compiler->addString( m_string );
-               m_string.exported( false );
-               return STRING;
+               uint32 nextChar;
+               if ( readAhead( nextChar ) && nextChar == '\'' )
+               {
+                  m_string.append( '\'' );
+                  m_in->discardReadAhead(1);
+               }
+               else {
+                  m_state = e_line;
+                  popContext();
+                  VALUE->stringp = m_compiler->addString( m_string );
+                  m_string.exported( false );
+                  return STRING;
+               }
             }
             else
                m_string.append( chr );
@@ -848,31 +907,10 @@ int SrcLexer::lex_normal()
             if ( chr == '\\' )
             {
                uint32 nextChar;
-               m_in->readAhead( nextChar );
+               readAhead( nextChar );
                switch ( nextChar )
                {
                   case '\\': nextChar = '\\'; break;
-                  case '\r':
-                     m_in->discardReadAhead( 1 );
-                     m_in->readAhead( nextChar );
-                     if( nextChar == '\n')
-                     {
-                        nextChar = ' ';
-                        m_previousLine = m_line;
-                        m_line++;
-                        m_character = 0;
-                        m_state = e_stringRunning;
-                     }
-                  break;
-
-                  case '\n':
-                     nextChar = ' ';
-                     m_previousLine = m_line;
-                     m_line++;
-                     m_character = 0;
-                     m_state = e_stringRunning;
-                  break;
-
                   case 'n': nextChar = '\n'; break;
                   case 'b': nextChar = '\b'; break;
                   case 't': nextChar = '\t'; break;
@@ -885,7 +923,13 @@ int SrcLexer::lex_normal()
                      m_state = e_stringHex;
                   break;
 
-                  case '0':
+                  case 'B':
+                     nextChar = 1;
+                     tempString.size(0);
+                     m_state = e_stringBin;
+                  break;
+
+                  case '0': case 'c': case 'C':
                      nextChar = 1;
                      tempString.size(0);
                      m_state = e_stringOctal;
@@ -901,7 +945,18 @@ int SrcLexer::lex_normal()
             }
             else if ( chr == '\n' )
             {
-               m_string.append( '\n' );
+               if( ! m_mlString )
+               {
+                  m_compiler->raiseError( e_nl_in_lit, m_line );
+                  m_lineFilled = false;
+                  m_bIsDirectiveLine = false;
+                  m_compiler->raiseError( e_nl_in_lit, m_previousLine );
+                  m_state = e_line;
+                  popContext();
+               }
+               else
+                  m_state = e_stringRunning;
+
                m_previousLine = m_line;
                m_line++;
                m_character = 0;
@@ -909,8 +964,11 @@ int SrcLexer::lex_normal()
             }
             else if ( chr == m_chrEndString )
             {
-               // closed string
-               m_state = e_postString;
+               popContext();
+               m_state = e_line;
+               VALUE->stringp = m_compiler->addString( m_string );
+               m_string.exported( false );
+               return STRING;
             }
             else
                m_string.append( chr );
@@ -925,92 +983,21 @@ int SrcLexer::lex_normal()
                   m_line++;
                   m_character = 0;
                }
+               else if ( chr == m_chrEndString )
+               {
+                  popContext();
+                  m_state = e_line;
+                  VALUE->stringp = m_compiler->addString( m_string );
+                  m_string.exported( false );
+                  return STRING;
+               }
                else {
                   m_state = e_string;
-                  m_string.append( chr );
-               }
-            }
-         break;
-
-         case e_postString:
-            if ( chr == '\n' ) {
-               // don't change previous line
-               m_line++;
-               m_character = 0;
-               m_addEol = true; // force to generate an EOL for the parser
-               m_bIsDirectiveLine = false;
-               // end of input?
-               if ( m_done ) {
-                  VALUE->stringp = m_compiler->addString( m_string );
-                  m_string.exported( false );
-                  m_state = e_line;
-                  return STRING;
-               }
-            }
-            else if ( chr == '"' )
-            {
-               // a chained string.
-               m_addEol = false;
-               m_state = e_string;
-               m_chrEndString = '"'; //... up to the matching "
-            }
-            else if ( chr == 0x201C )
-            {
-               // a chained string.
-               m_addEol = false;
-               m_state = e_string;
-               m_chrEndString = 0x201D; //... up to the matching close quote
-            }
-            else if ( chr == 0x300C )
-            {
-               // a chained string.
-               m_addEol = false;
-               m_state = e_string;
-               m_chrEndString = 0x300D; //... up to the matching close japanese quote
-            }
-            else if ( chr == '/' )
-            {
-               // may be a comment.
-               uint32 chr1;
-               if ( ! m_in->get( chr1 ) )
-               {
-                  // end of input.
-                  m_done = true;
-                  VALUE->stringp = m_compiler->addString( m_string );
-                  m_string.exported( false );
-                  return STRING;
-               }
-
-               if ( chr1 == '*' )
-               {
-                  m_state = e_blockCommentString;
-               }
-               else if ( chr1 == '/' ) {
-                  VALUE->stringp = m_compiler->addString( m_string );
-                  m_string.exported( false );
-                  m_state = e_eolComment;
-                  return STRING;
-               }
-               else {
-                  // not a comment
-                  m_in->unget( chr1 );
+                  if ( m_string.size() != 0 )
+                     m_string.append( ' ' );
                   m_in->unget( chr );
-                  VALUE->stringp = m_compiler->addString( m_string );
-                  m_string.exported( false );
-                  m_state = e_line;
-                  return STRING;
                }
             }
-            else if ( ! isWhiteSpace( chr ) )
-            {
-               m_in->unget( chr );
-               VALUE->stringp = m_compiler->addString( m_string );
-               m_string.exported( false );
-               m_state = e_line;
-               return STRING;
-            }
-
-
          break;
 
          case e_stringHex:
@@ -1021,7 +1008,7 @@ int SrcLexer::lex_normal()
             {
                   tempString.append( chr );
             }
-            else
+            else if ( chr != '_' )
             {
                m_in->unget( chr );
                uint64 retval;
@@ -1032,12 +1019,28 @@ int SrcLexer::lex_normal()
             }
          break;
 
+         case e_stringBin:
+            if ( chr == '0' || chr == '1' )
+            {
+               tempString.append( chr );
+            }
+            else if ( chr != '_' )
+            {
+               m_in->unget( chr );
+               uint64 retval;
+               if ( ! tempString.parseBin( retval ) || retval > 0xFFFFFFFF )
+                  m_compiler->raiseError( e_inv_esc_sequence, m_line );
+               m_string.append( (uint32) retval );
+               m_state = e_string;
+            }
+         break;
+
          case e_stringOctal:
             if (  (chr >= '0' && chr <= '7') )
             {
-                  tempString.append( chr );
+               tempString.append( chr );
             }
-            else
+            else if ( chr != '_' )
             {
                m_in->unget( chr );
                uint64 retval;
@@ -1062,12 +1065,16 @@ int SrcLexer::lex_normal()
 
 void SrcLexer::checkContexts()
 {
-   if ( m_contexts != 0 )
-      m_compiler->raiseContextError( e_par_unbal, m_line, m_ctxOpenLine );
-   if ( m_squareContexts != 0 )
-      m_compiler->raiseContextError( e_square_unbal, m_line, m_ctxOpenLine );
-   if ( m_state == e_string || m_state == e_litString )
-      m_compiler->raiseContextError( e_unclosed_string, m_line, m_ctxOpenLine );
+   t_contextType ct = currentContext();
+   
+   if ( ct == ct_round )
+      m_compiler->raiseContextError( e_par_unbal, m_line, contextStart() );
+   else if ( ct == ct_square )
+      m_compiler->raiseContextError( e_square_unbal, m_line, contextStart() );
+   else if ( ct == ct_graph )
+      m_compiler->raiseContextError( e_graph_unbal, m_line, contextStart() );
+   else if ( ct == ct_string  )
+      m_compiler->raiseContextError( e_unclosed_string, m_line, contextStart() );
 }
 
 
@@ -1081,7 +1088,7 @@ int SrcLexer::state_line( uint32 chr )
 
       // a real EOL has been provided here.
       m_bIsDirectiveLine = false;
-      if ( m_lineFilled && m_contexts == 0 && m_squareContexts == 0 )
+      if ( m_lineFilled && ! inParCtx() )
       {
          m_lineFilled = false;
          return EOL;
@@ -1091,7 +1098,7 @@ int SrcLexer::state_line( uint32 chr )
    {
       // don't return at next eol:
       uint32 nextChr;
-      if ( ! m_in->readAhead( nextChr ) )
+      if ( ! readAhead( nextChr ) )
       {
          // end of file; if we're in incremental mode,
          // declare the opening of a temporary context
@@ -1107,19 +1114,6 @@ int SrcLexer::state_line( uint32 chr )
          m_character = 0;
          m_in->discardReadAhead( 1 );
       }
-      else if ( nextChr == '\r' )
-      {
-         // discard next char
-         m_in->discardReadAhead( 1 );
-         m_in->readAhead( nextChr );
-         if ( nextChr == '\n' )
-         {
-            m_previousLine = m_line;
-            m_line++;
-            m_character = 0;
-            m_in->discardReadAhead( 2 );
-         }
-      }
       else if ( nextChr == '\\' )
       {
          m_in->discardReadAhead( 1 );
@@ -1178,26 +1172,76 @@ int SrcLexer::state_line( uint32 chr )
    }
    else if ( chr == '"' )
    {
+      uint32 nextChr;
       // we'll begin to read a string.
-      m_state = e_string;
-      m_ctxOpenLine = m_line;
+      if ( readAhead( nextChr ) && nextChr == '\n' )
+      {
+         m_mlString = true;
+         m_line++;
+         m_character = 0;
+         m_in->discardReadAhead( 1 );
+         m_state = e_stringRunning;
+      }
+      else {
+         m_mlString = false;
+         m_state = e_string;
+      }
+
+      pushContext( ct_string, m_line );
       m_chrEndString = '"'; //... up to the matching "
    }
    else if ( chr == 0x201C )
    {
+      uint32 nextChr;
+      // we'll begin to read a string.
+      if ( readAhead( nextChr ) && nextChr == '\n' )
+      {
+         m_mlString = true;
+         m_line++;
+         m_character = 1;
+         m_in->discardReadAhead( 1 );
+      }
+      else
+         m_mlString = false;
       // we'll begin to read a string.
       m_state = e_string;
+      pushContext( ct_string, m_line );
       m_chrEndString = 0x201D; //... up to the matching close quote
    }
    else if ( chr == 0x300C )
    {
+      uint32 nextChr;
+      // we'll begin to read a string.
+      if ( readAhead( nextChr ) && nextChr == '\n' )
+      {
+         m_mlString = true;
+         m_line++;
+         m_character = 1;
+         m_in->discardReadAhead( 1 );
+      }
+      else
+         m_mlString = false;
       // we'll begin to read a string.
       m_state = e_string;
+      pushContext( ct_string, m_line );
       m_chrEndString = 0x300D; //... up to the matching close japanese quote
    }
    else if ( chr == '\'' )
    {
+      uint32 nextChr;
+      // we'll begin to read a string.
+      if ( readAhead( nextChr ) && nextChr == '\n' )
+      {
+         m_mlString = true;
+         m_line++;
+         m_character = 1;
+         m_in->discardReadAhead( 1 );
+      }
+      else
+         m_mlString = false;
+
       // we'll begin to read a non escaped string
+      pushContext( ct_string, m_line );
       m_state = e_litString;
    }
    else if ( chr == '0' )
@@ -1237,7 +1281,7 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
          {
             m_bIsDirectiveLine = false;
             // but not first sym
-            if ( m_lineFilled )
+            if ( m_lineFilled && ! inParCtx() )
             {
                m_lineFilled = false;
                return EOL;
@@ -1296,41 +1340,52 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
          }
          else if ( chr == '(' || chr == 0xff08 )
          {
-            m_contexts++;
-            if ( m_contexts == 1 )
-               m_ctxOpenLine = m_line;
-
+            pushContext( ct_round, m_line );
             return OPENPAR;
          }
          else if ( chr == ')' || chr == 0xff09 )
          {
-            if ( m_contexts == 0 )
+            if ( currentContext() != ct_round )
                m_compiler->raiseError( e_par_close_unbal, m_line );
             else
             {
-               m_contexts--;
+               popContext();
                return CLOSEPAR;
             }
          }
          else if ( chr == '[' )
          {
-            m_squareContexts++;
-            if ( m_squareContexts == 1 )
-               m_ctxOpenLine = m_line;
+            pushContext( ct_square, m_line );
             return OPENSQUARE;
          }
          else if ( chr == ']' )
          {
-            if ( m_squareContexts == 0 )
+            if ( currentContext() != ct_square )
             {
                m_compiler->raiseError( e_square_close_unbal, m_line );
             }
             else
             {
-               m_squareContexts--;
+               popContext();
                return CLOSESQUARE;
             }
          }
+         else if ( chr == '{' )
+         {
+            pushContext( ct_graph, m_line );
+            return OPEN_GRAPH;
+         }
+         else if ( chr == '}' )
+         {
+            if ( currentContext() == ct_graph )
+            {
+               m_graphAgain = true;
+               popContext();
+               return EOL;
+            }
+            else 
+               m_compiler->raiseError( e_graph_close_unbal, m_line );
+         }
          else if ( chr == '@' )
          {
             return ATSIGN;
@@ -1405,18 +1460,9 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
             return FORDOT;
          else if ( m_string == ".[" )
          {
-            m_squareContexts++;
-            if ( m_squareContexts == 1 )
-               m_ctxOpenLine = m_line;
+            pushContext( ct_square, m_line );
             return LISTPAR;
          }
-         else if ( m_string == ".\"" )
-         {
-            // starting a string context
-            m_state = e_string;
-            m_chrEndString = '"'; //... up to the matching "
-            return 0;
-         }
          else if ( parsingFtd() && m_string == "?>" && (nextChar != '\n' || m_in->eof() ))
          {
             m_mode = t_mOutscape;
@@ -1432,6 +1478,9 @@ int SrcLexer::checkUnlimitedTokens( uint32 nextChar )
             if ( m_bWasntEmpty )
                m_whiteLead = "\n";
             m_bIsDirectiveLine = false;
+            m_previousLine = m_line;
+            m_line++;
+            m_character = 0;
             return EOL;
          }
          else if ( m_string == ">>=" )
@@ -1472,8 +1521,6 @@ int SrcLexer::checkLimitedTokens()
             return NIL;
          else if ( m_string == "for" )
             return FOR;
-         else if ( m_string == "has" )
-            return HAS;
          else if ( m_string == "and" )
             return AND;
          else if ( m_string == "and" )
@@ -1491,8 +1538,6 @@ int SrcLexer::checkLimitedTokens()
             m_bIsDirectiveLine = true;
             return LOAD;
          }
-         if ( m_string == "give" )
-            return GIVE;
          if ( m_string == "init" )
             return INIT;
          if ( m_string == "else" )
@@ -1503,8 +1548,6 @@ int SrcLexer::checkLimitedTokens()
             return FROM;
          if ( m_string == "self" )
             return SELF;
-         if ( m_string == "pass" )
-            return PASS;
          if ( m_string == "case" )
             return CASE;
          if ( m_string == "loop" )
@@ -1526,14 +1569,14 @@ int SrcLexer::checkLimitedTokens()
             return CLASS;
          if ( m_string == "notin" )
             return OP_NOTIN;
-         if ( m_string == "hasnt" )
-            return HASNT;
          if ( m_string == "const" )
             return CONST_KW;
          if ( m_string == "while" )
             return WHILE;
          if ( m_string == "false" )
             return FALSE_TOKEN;
+         if ( m_string == "fself" )
+            return FSELF;
          if( m_string == "macro" )
          {
             m_string.size(0);
@@ -1551,10 +1594,6 @@ int SrcLexer::checkLimitedTokens()
             return GLOBAL;
          if ( m_string == "launch" )
             return LAUNCH;
-         if ( m_string == "lambda" )
-            return LAMBDA;
-         if ( m_string == "sender" )
-            return SENDER;
          if ( m_string == "object" )
             return OBJECT;
          if ( m_string == "return" )
@@ -1606,12 +1645,6 @@ int SrcLexer::checkLimitedTokens()
          if ( m_string == "formiddle" )
             return FORMIDDLE;
       break;
-
-      case 10:
-
-         if ( m_string == "attributes" )
-            return ATTRIBUTES;
-      break;
    }
 
    return 0;
@@ -1633,11 +1666,18 @@ void SrcLexer::parsingFtd( bool b )
 
 void SrcLexer::resetContexts()
 {
+   // clear contexts
+   while( m_topCtx != 0 )
+   {
+      Context* ctx = m_topCtx->m_prev;
+      delete m_topCtx;
+      m_topCtx = ctx;
+   }
+   
    // force to generate a fake eol at next loop
    m_addEol = true;
    m_bIsDirectiveLine = false;
-   m_contexts = 0;
-   m_squareContexts = 0;
+   
    m_state = e_line;
    m_lineFilled = false;
    m_string = "";
@@ -1837,6 +1877,60 @@ void SrcLexer::parseMacroCall()
    }
 }
 
+
+void SrcLexer::pushContext( t_contextType ct, int startLine )
+{
+   m_topCtx = new Context( ct, startLine, m_topCtx );
+}
+
+
+bool SrcLexer::popContext()
+{
+   if( m_topCtx == 0 )
+      return false;
+   
+   Context* current = m_topCtx;
+   m_topCtx = m_topCtx->m_prev;
+   delete current;
+   return true;
+}
+
+
+SrcLexer::t_contextType SrcLexer::currentContext()
+{
+   if( m_topCtx == 0 )
+      return ct_top;
+   
+   return m_topCtx->m_ct;
+}
+
+   
+int SrcLexer::contextStart()
+{
+   if( m_topCtx == 0 )
+      return 0;
+   
+   return m_topCtx->m_oline;
+}
+
+bool SrcLexer::inParCtx()
+{
+   return m_topCtx != 0 &&  
+         ( m_topCtx->m_ct == ct_round || m_topCtx->m_ct == ct_square );
+}
+
+bool SrcLexer::readAhead( uint32 &chr )
+{
+   bool res;
+
+   while( (res = m_in->readAhead( chr )) && chr == '\r' )
+   {
+      m_in->discardReadAhead( 1 );
+   }
+
+   return res;
+}
+
 }
 
 
