@@ -17,7 +17,6 @@
 #define FALCON_COMPILER_H
 
 #include <falcon/syntree.h>
-#include <falcon/errhand.h>
 #include <falcon/error.h>
 #include <falcon/common.h>
 #include <falcon/string.h>
@@ -113,17 +112,6 @@ protected:
    SrcLexer *m_lexer;
    Stream *m_stream;
 
-   /** Error handler.
-      This interface is invoked to present the errors to the embedding application;
-      If an embedding VM is present (\see m_vm) this field is ignored, and the error
-      is sent directly to the VM for handling. If neither this field nor a virtual
-      machine are present, the error data printed on the error stream.
-
-      \note Probably, the ability to print on the error stream if error handler is not found
-      will be removed.
-   */
-   ErrorHandler *m_errhand;
-
    /** This is the module that is being formed in the meanwhile. */
    Module *m_module;
 
@@ -183,13 +171,16 @@ protected:
 
    bool m_bParsingFtd;
 
-   /** True to raise only one exception. */
-   bool m_delayRaise;
+   bool m_bInteractive;
+
    Error *m_rootError;
    InteractiveCompiler *m_metacomp;
    VMachine *m_serviceVM;
    ModuleLoader *m_serviceLoader;
 
+   /** Search path inherited from upper facilities. */
+   String m_searchPath;
+
    /** Removes all the structures and temporary data used to compile a file.
       This function is called automatically by the various compile() and
       destructors.
@@ -263,7 +254,20 @@ public:
    */
    bool compile( Module *mod, Stream *input );
 
+   /** Front-end to raiseError( *e ).
+
+      The compiler doesn't throw the error list until the compilation is over.
+      error raisal is delayed until the end of the compilation step.
+   */
    void raiseError( int errorNum, int errorLine=0);
+
+   /** Raises an error.
+
+      The compiler doesn't throw the error list until the compilation is over.
+      error raisal is delayed until the end of the compilation step.
+   */
+   void raiseError( Error *e );
+
    /** Raises an error related to a context problem.
       The error reports the line where the problem has been detected, and the line
       that begun current faulty context.
@@ -275,12 +279,28 @@ public:
    void raiseError( int errorNum, const String &errorp, int errorLine=0);
    void addError() { m_errors++; }
 
-   Symbol *searchLocalSymbol( const String *symname );
+   /** Searches a symbol in the local context.
+    *
+    *  If the current context is the global context, then it just calls
+    *  searchGlobalSymbol. If there is a local context, the symbol is serched
+    *  in the current context; in case it's not found, this method returns
+    *  if bRecurse is false.
+    *
+    *  If bRecurse is true, the symbol is searched down in the parent contexts
+    *  until found or until there is no more local context. In that case, the
+    *  method returns 0; so if there is the need to bind a local symbol or a global
+    *  one if local symbols are not found, this must be done by the caller
+    *  calling searchGlobalSymbol when this method returns 0.
+    *
+    *  \param symname The symbol name as a pointer to a module string.
+    *  \param bRecurse true to bind symbols from any parent, false to search in the local context.
+    *  \return The symbol if found or 0 otherwise.
+    */
+   Symbol *searchLocalSymbol( const String *symname, bool bRecurse = false );
    Symbol *searchGlobalSymbol( const String *symname );
    Symbol *addLocalSymbol( const String *symname, bool parameter );
    Symbol *addGlobalSymbol( const String *symname );
    Symbol *searchOuterSymbol( const String *symname );
-   Symbol *addAttribute( const String *symname );
 
    /** Creates a symbol that will be an initially defined global variable.
       The global variables may be created with an initial values (i.e. for
@@ -336,9 +356,6 @@ public:
 
    String *addString( const String &str ) { return m_module->addString( str ); }
 
-   void errorHandler( ErrorHandler *errhand ) { m_errhand = errhand; }
-   ErrorHandler *errorHandler() const { return m_errhand; }
-
    SrcLexer *lexer() const { return m_lexer; }
    Stream *stream() const { return m_stream; }
 
@@ -350,16 +367,9 @@ public:
    void addNumConstant( const String &name, numeric value, uint32 line=0 );
    void addStringConstant( const String &name, const String &value, uint32 line=0 );
    void addConstant( const String &name, Value *val, uint32 line=0 );
-/*
-   void Compiler::addConstant( const char *name, SyntreeElement *value );
 
-   SyntreeElement *Compiler::findConstant( const char *name ) const
-   {
-      tConstMap::const_iterator ci =  m_constants.find( name );
-      if ( ci != m_constants.end() )
-         return ci->second;
-      return 0;
-   }*/
+   /** Adds an attribute to the currently active context-sensible symbol */
+   void addAttribute( const String &name, Value *val, uint32 line=0 );
 
 
    SourceTree *sourceTree() const { return m_root; }
@@ -432,12 +442,6 @@ public:
    void strictMode( bool breq ) { m_strict = breq; }
    bool strictMode() const { return m_strict; }
 
-   /** Set to true to send all the errors to a list, and have only one error raised at the end. */
-   void delayRaise( bool setting ) { m_delayRaise = setting; }
-
-   /** Returns true if delay raise is enabled. */
-   bool delayRaise() const { return m_delayRaise; }
-
    /** Are we parsing a normal file or an escaped template file? */
    bool parsingFtd() const;
    void parsingFtd( bool b );
@@ -568,6 +572,42 @@ public:
       is requried, the compiler creates a loader on the fly.
    */
    void serviceLoader(ModuleLoader *l) { m_serviceLoader = l; }
+
+   /** Passes the ownership of the error structure to the caller.
+
+      This allows the caller to get the errors received during the processing of the
+      last compilation and handle them separately.
+
+      The ownership is passed to the caller, or in other world, the list of errors
+      in this compiler is zeroed and the reference count of the returned error
+      list is not changed.
+
+      \return 0 if the compiler was not in error state, or a list of one or more errors if it
+      raised some errors during the processing of the files.
+   */
+   Error *detachErrors() { Error *e = m_rootError; m_rootError = 0; m_errors = 0; return e; }
+
+   /** Sets this compiler as interactive.
+      A compiler meant to run code from the command line has different
+      rules; in example, it can accept autoexpressions without raising
+      a "statement does nothing" error, as the meaning of expressions
+      on the command line is just that to be evaluated and their result
+      being interactively displayed.
+   */
+   void setInteractive( bool bint ) { m_bInteractive = bint; }
+
+   /** Checks if this compiler is as an interactive I/O */
+   bool isInteractive() const { return m_bInteractive; }
+
+   /** Return the search path inherited from upper facilities.
+      This search path is used to drive module loading in macro compiler.
+   */
+   const String& searchPath() const { return m_searchPath; }
+
+   /** Sets the compiler-specific search path.
+      This search path is used to drive module loading in macro compiler.
+   */
+   void searchPath( const String& path ) { m_searchPath = path; }
 };
 
 } // end of namespace
